/*
 * GrammarCompare.h
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#ifndef GRAMMAR_COMPARE_H_
#define GRAMMAR_COMPARE_H_

#include <ostream>

#include <grammar/Grammar.h>
#include <set>
#include <list>
#include <map>
#include <utility>

class GrammarCompare : public grammar::VisitableGrammarBase::const_same_visitor_type {
public:
	GrammarCompare() {}

private:
	static bool testCompare(const grammar::LeftLG& a, const grammar::LeftLG& b);
	static void printCompare(const grammar::LeftLG& a, const grammar::LeftLG& b);

	static bool testCompare(const grammar::LeftRG& a, const grammar::LeftRG& b);
	static void printCompare(const grammar::LeftRG& a, const grammar::LeftRG& b);

	static bool testCompare(const grammar::RightLG& a, const grammar::RightLG& b);
	static void printCompare(const grammar::RightLG& a, const grammar::RightLG& b);

	static bool testCompare(const grammar::RightRG& a, const grammar::RightRG& b);
	static void printCompare(const grammar::RightRG& a, const grammar::RightRG& b);

	static bool testCompare(const grammar::LG& a, const grammar::LG& b);
	static void printCompare(const grammar::LG& a, const grammar::LG& b);

	static bool testCompare(const grammar::CFG& a, const grammar::CFG& b);
	static void printCompare(const grammar::CFG& a, const grammar::CFG& b);

	static bool testCompare(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b);
	static void printCompare(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b);

	static bool testCompare(const grammar::CNF& a, const grammar::CNF& b);
	static void printCompare(const grammar::CNF& a, const grammar::CNF& b);

	static bool testCompare(const grammar::GNF& a, const grammar::GNF& b);
	static void printCompare(const grammar::GNF& a, const grammar::GNF& b);

	static bool testCompare(const grammar::CSG& a, const grammar::CSG& b);
	static void printCompare(const grammar::CSG& a, const grammar::CSG& b);

	static bool testCompare(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b);
	static void printCompare(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b);

	static bool testCompare(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b);
	static void printCompare(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b);

	static bool testCompare(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b);
	static void printCompare(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b);

	template <class T> static void setCompare(const std::set<T> a, const std::set<T> b);
	template <class T> static void listCompare(const std::list<T> a, const std::list<T> b);
	template <class T, class R> static void mapCompare(const std::map<T, R> a, const std::map<T, R> b);
public:
	void Visit(void*, const grammar::RightRG& first, const grammar::RightRG& second) const;
	void Visit(void*, const grammar::LeftRG& first, const grammar::LeftRG& second) const;
	void Visit(void*, const grammar::RightLG& first, const grammar::RightLG& second) const;
	void Visit(void*, const grammar::LeftLG& first, const grammar::LeftLG& second) const;

	void Visit(void*, const grammar::LG& first, const grammar::LG& second) const;
	void Visit(void*, const grammar::CFG& first, const grammar::CFG& second) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& first, const grammar::EpsilonFreeCFG& second) const;
	void Visit(void*, const grammar::CNF& first, const grammar::CNF& second) const;
	void Visit(void*, const grammar::GNF& first, const grammar::GNF& second) const;

	void Visit(void*, const grammar::CSG& first, const grammar::CSG& second) const;
	void Visit(void*, const grammar::NonContractingGrammar& first, const grammar::NonContractingGrammar& second) const;

	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& first, const grammar::ContextPreservingUnrestrictedGrammar& second) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& first, const grammar::UnrestrictedGrammar& second) const;

	static int compare(const grammar::LeftLG& a, const grammar::LeftLG& b);
	static int compare(const grammar::LeftRG& a, const grammar::LeftRG& b);
	static int compare(const grammar::RightLG& a, const grammar::RightLG& b);
	static int compare(const grammar::RightRG& a, const grammar::RightRG& b);
	static int compare(const grammar::LG& a, const grammar::LG& b);
	static int compare(const grammar::CFG& a, const grammar::CFG& b);
	static int compare(const grammar::EpsilonFreeCFG& a, const grammar::EpsilonFreeCFG& b);
	static int compare(const grammar::CNF& a, const grammar::CNF& b);
	static int compare(const grammar::GNF& a, const grammar::GNF& b);
	static int compare(const grammar::CSG& a, const grammar::CSG& b);
	static int compare(const grammar::NonContractingGrammar& a, const grammar::NonContractingGrammar& b);
	static int compare(const grammar::ContextPreservingUnrestrictedGrammar& a, const grammar::ContextPreservingUnrestrictedGrammar& b);
	static int compare(const grammar::UnrestrictedGrammar& a, const grammar::UnrestrictedGrammar& b);

	static int compare(const grammar::Grammar& a, const grammar::Grammar& b);

	static const GrammarCompare GRAMMAR_COMPARE;
};

#endif /* GRAMMAR_COMPARE_H_ */
