/*
 * acompare.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>

#include "factory/XmlDataFactory.hpp"
#include "exception/AlibException.h"

#include "AutomatonCompare.h"
#include "GrammarCompare.h"

int main(int argc, char** argv) {

	try {

		TCLAP::CmdLine cmd("Regexp derivative compute binary", ' ', "0.01");

		TCLAP::UnlabeledValueArg<std::string> input1(	"input1",	"The first input to compare",	true,	"-",		"file");
		cmd.add( input1 );

		TCLAP::UnlabeledValueArg<std::string> input2(	"input2",	"The second input to compare",	true,	"-",		"file");
		cmd.add( input2 );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);

		std::deque<sax::Token> tokens1;
		if(input1.isSet()) {
			if(input1.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens1);
			} else {
				sax::SaxParseInterface::parseFile(input1.getValue(), tokens1);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens1);
		}

		std::deque<sax::Token> tokens2;
		if(input2.isSet()) {
			if(input2.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens2);
			} else {
				sax::SaxParseInterface::parseFile(input2.getValue(), tokens2);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens2);
		}

		if (tokens1.empty() && tokens2.empty()) throw exception::AlibException("Invalid arguments tokens streams 1 and 2 are empty.");
		if (tokens1.empty()) throw exception::AlibException("Invalid arguments tokens stream 1 is empty.");
		if (tokens2.empty()) throw exception::AlibException("Invalid arguments tokens stream 2 is empty.");

		if(alib::XmlDataFactory::first<exception::AlibException>(tokens1)) std::cerr << alib::XmlDataFactory::fromTokens<exception::AlibException>(tokens1).getCause() << std::endl;
		if(alib::XmlDataFactory::first<exception::AlibException>(tokens2)) std::cerr << alib::XmlDataFactory::fromTokens<exception::AlibException>(tokens2).getCause() << std::endl;
		if(alib::XmlDataFactory::first<exception::AlibException>(tokens1) || alib::XmlDataFactory::first<exception::AlibException>(tokens2)) return 0;

		int res;

		if(alib::XmlDataFactory::first<automaton::Automaton>(tokens1) && alib::XmlDataFactory::first<automaton::Automaton>(tokens2)) {
			automaton::Automaton automaton1 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens1);
			automaton::Automaton automaton2 = alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens2);

			std::chrono::measurements::end();
			std::chrono::measurements::start("Compare", std::chrono::measurements::Type::MAIN);

			res = AutomatonCompare::compare(automaton1, automaton2);
		} else if(alib::XmlDataFactory::first<grammar::Grammar>(tokens1) && alib::XmlDataFactory::first<grammar::Grammar>(tokens2)) {
			grammar::Grammar grammar1 = alib::XmlDataFactory::fromTokens<grammar::Grammar>(tokens1);
			grammar::Grammar grammar2 = alib::XmlDataFactory::fromTokens<grammar::Grammar>(tokens2);

			std::chrono::measurements::end();
			std::chrono::measurements::start("Compare", std::chrono::measurements::Type::MAIN);

			res = GrammarCompare::compare(grammar1, grammar2);
		} else {
			throw exception::AlibException("Only automata and grammars can be compared.");
		}

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		return res;
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
