/*
 * aderivation.cpp
 *
 *  Created on: 19. 1. 2014
 *      Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/AlibException.h>

#include "regexp/transform/RegExpDerivation.h"

int main(int argc, char** argv) {
	try {
		TCLAP::CmdLine cmd("Regexp derivative compute binary", ' ', "0.01");
		TCLAP::ValueArg<std::string> string(	"s",	"string",	"String to derivate by",	false,	"-",		"file");
		cmd.add( string );

		TCLAP::ValueArg<std::string> regexp(	"r",	"regexp",	"Regexp to derivate",		false,	"-",		"file");
		cmd.add( regexp );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);

		std::deque<sax::Token> stringTokens;
		if(string.isSet()) {
			if(string.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(stringTokens);
			} else {
				sax::SaxParseInterface::parseFile(string.getValue(), stringTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(stringTokens);
		}

		std::deque<sax::Token> regexpTokens;
		if(regexp.isSet()) {
			if(regexp.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(regexpTokens);
			} else {
				sax::SaxParseInterface::parseFile(regexp.getValue(), regexpTokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(regexpTokens);
		}

		string::LinearString stringData = alib::XmlDataFactory::fromTokens<string::LinearString>(stringTokens);
		regexp::RegExp regexpData = alib::XmlDataFactory::fromTokens<regexp::RegExp>(regexpTokens);

		std::chrono::measurements::end();
		std::chrono::measurements::start("Algorithm", std::chrono::measurements::Type::MAIN);

		regexp::RegExp result = regexp::RegExpDerivation::derivation(regexpData, stringData);

		std::chrono::measurements::end();
		std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

		alib::XmlDataFactory::toStdout(result);

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		if(measure.getValue()) std::clog << std::chrono::measurements::results() << std::endl;

		return 0;
	} catch(const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cerr << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
