/*
 * ExactSubtreeMatch.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_SUBTREE_MATCH_H_
#define _EXACT_SUBTREE_MATCH_H_

#include <tree/Tree.h>
#include <tree/ranked/RankedNode.h>
#include <tree/unranked/UnrankedNode.h>
#include <set>
#include <common/multipleDispatch.hpp>

namespace arbology {

namespace exact {

class ExactSubtreeMatch : public std::PromotingDoubleDispatch<std::set<unsigned>, tree::TreeBase> {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set<unsigned> match(const tree::Tree& subject, const tree::Tree& pattern);

	static std::set<unsigned> match(const tree::UnrankedTree& subject, const tree::UnrankedTree& pattern);
	static std::set<unsigned> match(const tree::RankedTree& subject, const tree::RankedTree& pattern);
	static std::set<unsigned> match(const tree::PrefixRankedTree& subject, const tree::PrefixRankedTree& pattern);
	static std::set<unsigned> match(const tree::PrefixRankedBarTree& subject, const tree::PrefixRankedBarTree& pattern);
private:
	static bool matchHelper(const tree::UnrankedNode& subject, const tree::UnrankedNode& pattern);
	static bool matchHelper(const tree::RankedNode& subject, const tree::RankedNode& pattern);

	static void matchInternal(unsigned& index, std::set<unsigned>& occ, const tree::UnrankedNode& subject, const tree::UnrankedNode& pattern);
	static void matchInternal(unsigned& index, std::set<unsigned>& occ, const tree::RankedNode& subject, const tree::RankedNode& pattern);

public:
	static ExactSubtreeMatch& getInstance() {
		static ExactSubtreeMatch res;
		return res;
	}
};

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_SUBTREE_MATCH_H_ */
