/*
 * ToGrammarRightRG.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Tomas Pecka
 */

#include "ToGrammarRightRG.h"
#include <alphabet/LabeledSymbol.h>
#include <exception/AlibException.h>

namespace automaton {

namespace convert {

grammar::Grammar ToGrammarRightRG::convert(const automaton::Automaton& automaton) {
	return getInstance().dispatch(automaton.getData());
}

grammar::RightRG ToGrammarRightRG::convert(const automaton::NFA& automaton) {
	std::map<automaton::State, alphabet::Symbol> nonterminalMap;

	const automaton::State& initState = automaton.getInitialState();
	const alphabet::Symbol initSymbol(alphabet::LabeledSymbol(initState.getName()));

	grammar::RightRG grammar(initSymbol);
	nonterminalMap.insert(std::make_pair(initState, initSymbol));

	grammar.setTerminalAlphabet(automaton.getInputAlphabet());

	for(const auto& state : automaton.getStates()) {
		if(state == initState)
			continue;

		alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(state.getName())), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		grammar.addNonterminalSymbol(nt);
		nonterminalMap.insert(std::make_pair(state, nt));
	}

	// step 2 - create set of P in G
	for(const auto& transition : automaton.getTransitions()) {
		const automaton::State& from = transition.first.first;
		const alphabet::Symbol& input = transition.first.second;
		for(const auto& to : transition.second)
		{
			grammar.addRule(nonterminalMap.find(from)->second, std::make_pair(input, nonterminalMap.find(to)->second)); // 2a
			if(automaton.getFinalStates().count(to)) // 2b
				grammar.addRule(nonterminalMap.find(from)->second, input);
		}
	}

	// step 3 - set start symbol of G
	grammar.setInitialSymbol(nonterminalMap.find(automaton.getInitialState())->second);

	// step 4
	if(automaton.getFinalStates().count(automaton.getInitialState()))
		grammar.setGeneratesEpsilon(true); // okay this feature breaks algorithm but simplifies the code actually :))

	return grammar;
}

auto ToGrammarRightRGNFA = ToGrammarRightRG::RegistratorWrapper<grammar::RightRG, automaton::NFA>(ToGrammarRightRG::getInstance(), ToGrammarRightRG::convert);

grammar::RightRG ToGrammarRightRG::convert(const automaton::DFA& automaton) {
	std::map<automaton::State, alphabet::Symbol> nonterminalMap;

	const automaton::State& initState = automaton.getInitialState();
	const alphabet::Symbol initSymbol(alphabet::LabeledSymbol(initState.getName()));

	grammar::RightRG grammar(initSymbol);
	nonterminalMap.insert(std::make_pair(initState, initSymbol));

	grammar.setTerminalAlphabet(automaton.getInputAlphabet());

	for(const auto& state : automaton.getStates()) {
		if(state == initState)
			continue;

		alphabet::Symbol nt = alphabet::createUniqueSymbol(alphabet::Symbol(alphabet::LabeledSymbol(state.getName())), grammar.getTerminalAlphabet(), grammar.getNonterminalAlphabet());
		grammar.addNonterminalSymbol(nt);
		nonterminalMap.insert(std::make_pair(state, nt));
	}

	// step 2 - create set of P in G
	for(const auto& transition : automaton.getTransitions()) {
		const automaton::State& from = transition.first.first;
		const alphabet::Symbol& input = transition.first.second;
		const automaton::State& to = transition.second;

		grammar.addRule(nonterminalMap.find(from)->second, std::make_pair(input, nonterminalMap.find(to)->second)); // 2a
		if(automaton.getFinalStates().count(to)) // 2b
			grammar.addRule(nonterminalMap.find(from)->second, input);
	}

	// step 3 - set start symbol of G
	grammar.setInitialSymbol(nonterminalMap.find(automaton.getInitialState())->second);

	// step 4
	if(automaton.getFinalStates().count(automaton.getInitialState()))
		grammar.setGeneratesEpsilon(true); // okay this feature breaks algorithm but simplifies the code actually :))

	return grammar;
}

auto ToGrammarRightRGDFA = ToGrammarRightRG::RegistratorWrapper<grammar::RightRG, automaton::DFA>(ToGrammarRightRG::getInstance(), ToGrammarRightRG::convert);

} /* namespace convert */

} /* namespace automaton */
