/*
 * Determinize.cpp
 *
 *  Created on: 16. 1. 2014
 *	  Author: Jan Vesely
 */

#include "Determinize.h"
#include "../transform/PDAToRHPDA.h"
#include "../transform/RHPDAToPDA.h"
#include "automaton/PDA/NPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicNPDA.h"

namespace automaton {

namespace determinize {

automaton::Automaton Determinize::determinize(const automaton::Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, Determinize::DETERMINIZE);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

void Determinize::Visit(void*, const automaton::EpsilonNFA&) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void Determinize::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->determinize(automaton));
}

void Determinize::Visit(void* data, const automaton::NFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->determinize(automaton));
}

void Determinize::Visit(void* data, const automaton::DFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void*, const automaton::ExtendedNFA& ) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void Determinize::Visit(void*, const automaton::CompactNFA& ) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

void Determinize::Visit(void* data, const automaton::DPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void* data, const automaton::SinglePopDPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void* data, const automaton::InputDrivenDPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void* data, const automaton::InputDrivenNPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->determinize(automaton));
}

void Determinize::Visit(void* data, const automaton::VisiblyPushdownDPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void* data, const automaton::VisiblyPushdownNPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->determinize(automaton));
}

void Determinize::Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->determinize(automaton));
}

void Determinize::Visit(void* data, const automaton::NPDA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	automaton::RealTimeHeightDeterministicNPDA rhpda = automaton::PDAToRHPDA::convert(automaton);
	automaton::RealTimeHeightDeterministicDPDA dpda = this->determinize(rhpda);
	out = new automaton::Automaton(automaton::RHPDAToPDA::convert(dpda));
}

void Determinize::Visit(void*, const automaton::SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void Determinize::Visit(void* data, const automaton::OneTapeDTM& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

void Determinize::Visit(void* data, const automaton::NFTA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->determinize(automaton));
}

void Determinize::Visit(void* data, const automaton::DFTA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(automaton);
}

const Determinize Determinize::DETERMINIZE;

} /* namespace determinize */

} /* namespace automaton */

#include "DeterminizeNFAPart.cxx"
#include "DeterminizeIDPDAPart.cxx"
#include "DeterminizeVPAPart.cxx"
#include "DeterminizeRHDPDAPart.cxx"
#include "DeterminizeNFTAPart.cxx"

