/*
 * EpsilonClosure.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef EPSILON_CLOSURE_H_
#define EPSILON_CLOSURE_H_

#include <algorithm>
#include <deque>
#include <set>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

namespace properties {

class EpsilonClosure : public VisitableAutomatonBase::const_visitor_type {
public:
	EpsilonClosure() {}
	static std::set<automaton::State> epsilonClosure( const automaton::Automaton & automaton, const automaton::State & state );

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static std::set<automaton::State> epsilonClosure( const automaton::EpsilonNFA & fsm, const automaton::State & state );
	static std::set<automaton::State> epsilonClosure( const automaton::MultiInitialStateNFA & fsm, const automaton::State & state );
	static std::set<automaton::State> epsilonClosure( const automaton::NFA & fsm, const automaton::State & state );
	static std::set<automaton::State> epsilonClosure( const automaton::DFA & fsm, const automaton::State & state );
	static std::set<automaton::State> epsilonClosure( const automaton::ExtendedNFA & fsm, const automaton::State & state );
	static std::set<automaton::State> epsilonClosure( const automaton::CompactNFA & fsm, const automaton::State & state );

private:
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;
	void Visit(void*, const DFTA& automaton) const;
	void Visit(void*, const NFTA& automaton) const;

	static const EpsilonClosure EPSILON_CLOSURE;
};

} /* namespace properties */

} /* namespace automaton */

#endif /* EPSILON_CLOSURE_H_ */
