/*
 * Accept.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_ACCEPT_H__
#define _AUTOMATON_ACCEPT_H__

#include <automaton/Automaton.h>
#include <object/Object.h>
#include <string/LinearString.h>
#include <tree/ranked/RankedTree.h>

namespace automaton {

namespace run {

class Accept : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	Accept() {}

	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static bool accept(const automaton::Automaton& automaton, const alib::Object& object);
	static bool accept(const automaton::Automaton& automaton, const string::LinearString& string);
	static bool accept(const automaton::Automaton& automaton, const tree::RankedTree& string);

	static bool accept(const automaton::DFA& automaton, const string::LinearString& string);
	static bool accept(const automaton::NFA& automaton, const string::LinearString& string);
	static bool accept(const automaton::DPDA& automaton, const string::LinearString& string);
	static bool accept(const automaton::DFTA& automaton, const tree::RankedTree& tree);
	static bool accept(const automaton::NFTA& automaton, const tree::RankedTree& tree);
private:
	static State calculateState(const automaton::DFTA& automaton, const tree::RankedNode & node);

	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;
	void Visit(void*, const DFTA& automaton) const;
	void Visit(void*, const NFTA& automaton) const;

	static const Accept ACCEPT;
};

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_ACCEPT_H__ */
