/*
 * MinimizeBrzozowski.cpp
 *
 *  Created on: 18. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "MinimizeBrzozowski.h"

#include <exception/AlibException.h>
#include <automaton/Automaton.h>
#include <automaton/FSM/MultiInitialStateNFA.h>

#include "../transform/Reverse.h"
#include "../determinize/Determinize.h"

namespace automaton {

namespace simplify {

automaton::Automaton MinimizeBrzozowski::minimize(const automaton::Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, MinimizeBrzozowski::MINIMIZE_BRZOZOWSKI);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

automaton::DFA MinimizeBrzozowski::minimize(const automaton::DFA& dfa) {
	return automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(dfa))));
}

automaton::DFA MinimizeBrzozowski::minimize(const automaton::NFA& nfa) {
	return automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(automaton::determinize::Determinize::determinize(automaton::transform::Reverse::convert(nfa))));
}

void MinimizeBrzozowski::Visit(void*, const automaton::EpsilonNFA&) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void MinimizeBrzozowski::Visit(void* data, const automaton::DFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->minimize(automaton));
}

void MinimizeBrzozowski::Visit(void*, const automaton::MultiInitialStateNFA&) const {
	throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
}

void MinimizeBrzozowski::Visit(void* data, const automaton::NFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->minimize(automaton));
}

void MinimizeBrzozowski::Visit(void*, const automaton::ExtendedNFA& ) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void MinimizeBrzozowski::Visit(void*, const automaton::CompactNFA& ) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

const MinimizeBrzozowski MinimizeBrzozowski::MINIMIZE_BRZOZOWSKI;

} /* namespace simplify */

} /* namespace automaton */
