/*
 * Rename.h
 *
 *  Created on: Dec 9, 2013
 *      Author: Jan Travnicek
 */

#ifndef RENAME_H_
#define RENAME_H_

#include <automaton/FSM/DFA.h>
#include <automaton/PDA/DPDA.h>
#include <automaton/PDA/SinglePopDPDA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/Automaton.h>

namespace automaton {

namespace simplify {

class Rename : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	Rename() {}

	/**
	 * @param dfa automaton to rename
	 */
	static automaton::Automaton rename(const automaton::Automaton& dfa);

	static automaton::DFA rename(const automaton::DFA& dfa);
	static automaton::DPDA rename(const automaton::DPDA& pda);
	static automaton::SinglePopDPDA rename(const automaton::SinglePopDPDA& pda);
	static automaton::InputDrivenDPDA rename(const automaton::InputDrivenDPDA& pda);
	static automaton::VisiblyPushdownDPDA rename(const automaton::VisiblyPushdownDPDA& pda);
	static automaton::RealTimeHeightDeterministicDPDA rename(const automaton::RealTimeHeightDeterministicDPDA& pda);


protected:
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;
	void Visit(void*, const automaton::DFTA& automaton) const;
	void Visit(void*, const automaton::NFTA& automaton) const;

	static const Rename RENAME;
};

} /* namespace simplify */

} /* namespace automaton */

#endif /* RENAME_H_ */
