/*
 * SingleInitialState.h
 *
 *  Created on: 20. 9. 2014
 *	  Author: Tomas Pecka
 */

#ifndef _SINGLE_INITIAL_STATE_H_
#define _SINGLE_INITIAL_STATE_H_

#include <automaton/Automaton.h>
#include <automaton/common/State.h>

namespace automaton {

namespace simplify {

/**
 * Makes finite automaton's transition function convert.
 * Source: Melichar: Algorithm 2.22
 */
class SingleInitialState : public VisitableAutomatonBase::const_visitor_type {
public:
	SingleInitialState() {}

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::Automaton convert(const automaton::Automaton& automaton);

	static automaton::NFA convert(const automaton::MultiInitialStateNFA& automaton);

	static automaton::DFA convert(const automaton::DFA& automaton);
	static automaton::NFA convert(const automaton::NFA& automaton);
	static automaton::EpsilonNFA convert(const automaton::EpsilonNFA& automaton);
	static automaton::ExtendedNFA convert(const automaton::ExtendedNFA& automaton);
	static automaton::CompactNFA convert(const automaton::CompactNFA& automaton);

private:
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DPDA& automaton) const;
	void Visit(void*, const SinglePopDPDA& automaton) const;
	void Visit(void*, const InputDrivenDPDA& automaton) const;
	void Visit(void*, const InputDrivenNPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;
	void Visit(void*, const DFTA& automaton) const;
	void Visit(void*, const NFTA& automaton) const;

	static const SingleInitialState SINGLE_INITIAL_STATE;
};

} /* namespace simplify */

} /* namespace automaton */

#endif /* _SINGLE_INITIAL_STATE_H_ */
