/*
 * Total.h
 *
 *  Created on: 20. 9. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TOTAL_H_
#define TOTAL_H_

#include <algorithm>
#include <deque>
#include <set>

#include "automaton/Automaton.h"
#include <automaton/common/State.h>

namespace automaton {

namespace simplify {

/**
 * Makes finite automaton's transition function total.
 * Source: Melichar: Algorithm 2.22
 */
class Total : public automaton::VisitableConstFSMBase {
public:
	Total() {}

	/**
	 * Computes epsilon closure of a state in epsilon nonfree automaton
	 */
	static automaton::Automaton total(const automaton::Automaton& automaton);

	static automaton::NFA total(const automaton::NFA& automaton);
	static automaton::DFA total(const automaton::DFA& automaton);

private:
	using automaton::VisitableConstFSMBase::Visit;

	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;

	static const Total TOTAL;
};

} /* namespace simplify */

} /* namespace automaton */

#endif /* TOTAL_H_ */
