/*
 * Trim.h
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATON_TRIM_H_
#define AUTOMATON_TRIM_H_

#include <algorithm>
#include <deque>
#include <set>
#include <automaton/Automaton.h>

namespace automaton {

namespace simplify {

class Trim : public automaton::VisitableAutomatonBase::const_visitor_type {
public:
	Trim() {}
	static automaton::Automaton trim( const automaton::Automaton & automaton );

	/**
	 * Removes dead states from FSM. Melichar 2.29
	 */
	template<class T>
	static T trim( const T & fsm );

private:
	void Visit(void*, const automaton::EpsilonNFA& automaton) const;
	void Visit(void*, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void*, const automaton::NFA& automaton) const;
	void Visit(void*, const automaton::DFA& automaton) const;
	void Visit(void*, const automaton::ExtendedNFA& automaton) const;
	void Visit(void*, const automaton::CompactNFA& automaton) const;
	void Visit(void*, const automaton::DPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenDPDA& automaton) const;
	void Visit(void*, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void*, const automaton::VisiblyPushdownNPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void*, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void*, const automaton::NPDA& automaton) const;
	void Visit(void*, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void*, const automaton::OneTapeDTM& automaton) const;
	void Visit(void*, const automaton::DFTA& automaton) const;
	void Visit(void*, const automaton::NFTA& automaton) const;

	static const Trim TRIM;
};

} /* namespace simplify */

} /* namespace automaton */

#endif /* AUTOMATON_TRIM_H_ */
