/*
 * AutomataUnionEpsilonTransition.h
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATA_UNION_EPSILON_H_
#define AUTOMATA_UNION_EPSILON_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/EpsilonNFA.h>

namespace automaton
{

namespace transform
{

/**
 * Union two automata.
 *  - For finite automata A1, A2, we create automaton L accepting L(A1) \cup L(A2)
 */
class AutomataUnionEpsilonTransition : public automaton::VisitableAutomatonBase::const_promoting_visitor_type
{
public:
	AutomataUnionEpsilonTransition() {}

	static automaton::Automaton unification(const automaton::Automaton& first, const automaton::Automaton& second);

	static automaton::EpsilonNFA unification(const automaton::EpsilonNFA& first, const automaton::EpsilonNFA& second);
	static automaton::EpsilonNFA unification(const automaton::NFA& first, const automaton::NFA& second);
	static automaton::EpsilonNFA unification(const automaton::DFA& first, const automaton::DFA& second);

private:
	void Visit(void* data, const automaton::CompactNFA& first, const automaton::CompactNFA& second) const;
	void Visit(void* data, const automaton::DFA& first, const automaton::DFA& second) const;
	void Visit(void* data, const automaton::EpsilonNFA& first, const automaton::EpsilonNFA& second) const;
	void Visit(void* data, const automaton::ExtendedNFA& first, const automaton::ExtendedNFA& second) const;
	void Visit(void* data, const automaton::MultiInitialStateNFA& first, const automaton::MultiInitialStateNFA& second) const;
	void Visit(void* data, const automaton::NFA& first, const automaton::NFA& second) const;

	void Visit(void* data, const automaton::DPDA& first, const automaton::DPDA& second) const;
	void Visit(void* data, const automaton::NPDA& first, const automaton::NPDA& second) const;
	void Visit(void* data, const automaton::InputDrivenDPDA& first, const automaton::InputDrivenDPDA& second) const;
	void Visit(void* data, const automaton::InputDrivenNPDA& first, const automaton::InputDrivenNPDA& second) const;
	void Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& first, const automaton::RealTimeHeightDeterministicDPDA& second) const;
	void Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& first, const automaton::RealTimeHeightDeterministicNPDA& second) const;
	void Visit(void* data, const automaton::SinglePopNPDA& first, const automaton::SinglePopNPDA& second) const;
	void Visit(void* data, const automaton::SinglePopDPDA& first, const automaton::SinglePopDPDA& second) const;
	void Visit(void* data, const automaton::VisiblyPushdownDPDA& first, const automaton::VisiblyPushdownDPDA& second) const;
	void Visit(void* data, const automaton::VisiblyPushdownNPDA& first, const automaton::VisiblyPushdownNPDA& second) const;

	void Visit(void* data, const automaton::OneTapeDTM& first, const automaton::OneTapeDTM& second) const;

	void Visit(void* data, const automaton::DFTA& first, const automaton::DFTA& second) const;
	void Visit(void* data, const automaton::NFTA& first, const automaton::NFTA& second) const;

	static const AutomataUnionEpsilonTransition AUTOMATA_UNION_EPSILON_TRANSITION;
};

} /* namespace transform */

} /* namespace automaton */

#endif /* AUTOMATA_UNION_EPSILON_H_ */
