/*
 * AutomatonIteration.cpp
 *
 *  Created on: 29. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "AutomatonIteration.h"
#include <exception/AlibException.h>

namespace automaton
{

namespace transform
{

automaton::Automaton AutomatonIteration::iteration(const automaton::Automaton& automaton)
{
	AutomatonBase* out;
	automaton.getData().Accept((void*) &out, AutomatonIteration::AUTOMATON_ITERATION);
	automaton::Automaton res(*out);
	delete out;
	return res;
}

template<class T>
automaton::NFA AutomatonIteration::iteration(const T& automaton)
{
	automaton::NFA res(automaton);

	for(const auto& qf : res.getFinalStates())
		for(const auto& t : res.getTransitionsToState(qf))
			res.addTransition(t.first.first, t.first.second, res.getInitialState());

	res.addFinalState(automaton.getInitialState());
	return res;
}

void AutomatonIteration::Visit(void*, const automaton::EpsilonNFA&) const
{
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void AutomatonIteration::Visit(void*, const automaton::CompactNFA&) const
{
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

void AutomatonIteration::Visit(void* data, const automaton::DFA& automaton) const
{
	AutomatonBase* &ret = *(AutomatonBase**) data;
	ret = std::move(AutomatonIteration::iteration(automaton)).plunder();
}

void AutomatonIteration::Visit(void*, const automaton::ExtendedNFA&) const
{
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void AutomatonIteration::Visit(void*, const automaton::MultiInitialStateNFA&) const
{
	throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
}

void AutomatonIteration::Visit(void* data, const automaton::NFA& automaton) const
{
	AutomatonBase* &ret = *(AutomatonBase**) data;
	ret = std::move(AutomatonIteration::iteration(automaton)).plunder();
}

void AutomatonIteration::Visit(void*, const automaton::DPDA&) const
{
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void AutomatonIteration::Visit(void*, const automaton::NPDA&) const
{
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void AutomatonIteration::Visit(void*, const automaton::InputDrivenDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void AutomatonIteration::Visit(void*, const automaton::InputDrivenNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void AutomatonIteration::Visit(void*, const automaton::RealTimeHeightDeterministicDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void AutomatonIteration::Visit(void*, const automaton::RealTimeHeightDeterministicNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void AutomatonIteration::Visit(void*, const automaton::SinglePopNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void AutomatonIteration::Visit(void*, const automaton::SinglePopDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void AutomatonIteration::Visit(void*, const automaton::VisiblyPushdownDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void AutomatonIteration::Visit(void*, const automaton::VisiblyPushdownNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void AutomatonIteration::Visit(void*, const automaton::OneTapeDTM&) const
{
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

void AutomatonIteration::Visit(void*, const automaton::DFTA&) const
{
	throw exception::AlibException("Unsupported automaton type DFTA");
}

void AutomatonIteration::Visit(void*, const automaton::NFTA&) const
{
	throw exception::AlibException("Unsupported automaton type NFTA");
}

const AutomatonIteration AutomatonIteration::AUTOMATON_ITERATION;

} /* namespace transform */

} /* namespace automaton */
