/*
 * AutomatonIteration.h
 *
 *  Created on: 29. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATON_ITERATION_H_
#define AUTOMATON_ITERATION_H_

#include <automaton/Automaton.h>
#include <automaton/FSM/EpsilonNFA.h>

namespace automaton
{

namespace transform
{

/**
 * Iterates language given by automaton
 *  - For finite automaton A1, we create automaton L accepting L(A1)*
 */
class AutomatonIteration : public automaton::VisitableAutomatonBase::const_visitor_type
{
public:
	AutomatonIteration() {}

	static automaton::Automaton iteration(const automaton::Automaton& automaton);

	template<class T>
	static automaton::NFA iteration(const T& automaton);

private:
	void Visit(void* data, const automaton::CompactNFA& automaton) const;
	void Visit(void* data, const automaton::DFA& automaton) const;
	void Visit(void* data, const automaton::EpsilonNFA& automaton) const;
	void Visit(void* data, const automaton::ExtendedNFA& automaton) const;
	void Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const;
	void Visit(void* data, const automaton::NFA& automaton) const;

	void Visit(void* data, const automaton::DPDA& automaton) const;
	void Visit(void* data, const automaton::NPDA& automaton) const;
	void Visit(void* data, const automaton::InputDrivenDPDA& automaton) const;
	void Visit(void* data, const automaton::InputDrivenNPDA& automaton) const;
	void Visit(void* data, const automaton::RealTimeHeightDeterministicDPDA& automaton) const;
	void Visit(void* data, const automaton::RealTimeHeightDeterministicNPDA& automaton) const;
	void Visit(void* data, const automaton::SinglePopNPDA& automaton) const;
	void Visit(void* data, const automaton::SinglePopDPDA& automaton) const;
	void Visit(void* data, const automaton::VisiblyPushdownDPDA& automaton) const;
	void Visit(void* data, const automaton::VisiblyPushdownNPDA& automaton) const;

	void Visit(void* data, const automaton::OneTapeDTM& automaton) const;

	void Visit(void* data, const automaton::DFTA& first) const;
	void Visit(void* data, const automaton::NFTA& first) const;

	static const AutomatonIteration AUTOMATON_ITERATION;
};

} /* namespace transform */

} /* namespace automaton */

#endif /* AUTOMATON_ITERATION_H_ */
