/*
 * AutomatonIterationEpsilonTransition.cpp
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "AutomatonIterationEpsilonTransition.h"
#include <exception/AlibException.h>

namespace automaton
{

namespace transform
{

automaton::Automaton AutomatonIterationEpsilonTransition::iteration(const automaton::Automaton& automaton)
{
	AutomatonBase* out;
	automaton.getData().Accept((void*) &out, AutomatonIterationEpsilonTransition::AUTOMATON_ITERATION_EPSILON_TRANSITION);
	automaton::Automaton res(*out);
	delete out;
	return res;
}

template<class T>
automaton::EpsilonNFA AutomatonIterationEpsilonTransition::iteration(const T& automaton)
{
	automaton::EpsilonNFA res(automaton);

	for(const auto&q : automaton.getFinalStates())
		res.addTransition(q, automaton.getInitialState());

	res.addFinalState(automaton.getInitialState());
	return res;
}

void AutomatonIterationEpsilonTransition::Visit(void* data, const automaton::EpsilonNFA& automaton) const
{
	AutomatonBase* &ret = *(AutomatonBase**) data;
	ret = std::move(AutomatonIterationEpsilonTransition::iteration(automaton)).plunder();
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::CompactNFA&) const
{
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

void AutomatonIterationEpsilonTransition::Visit(void* data, const automaton::DFA& automaton) const
{
	AutomatonBase* &ret = *(AutomatonBase**) data;
	ret = std::move(AutomatonIterationEpsilonTransition::iteration(automaton)).plunder();
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::ExtendedNFA&) const
{
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::MultiInitialStateNFA&) const
{
	throw exception::AlibException("Unsupported automaton type MultiInitialStateNFA");
}

void AutomatonIterationEpsilonTransition::Visit(void* data, const automaton::NFA& automaton) const
{
	AutomatonBase* &ret = *(AutomatonBase**) data;
	ret = std::move(AutomatonIterationEpsilonTransition::iteration(automaton)).plunder();
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::DPDA&) const
{
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::NPDA&) const
{
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::InputDrivenDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::InputDrivenNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::RealTimeHeightDeterministicDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::RealTimeHeightDeterministicNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::SinglePopNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::SinglePopDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::VisiblyPushdownDPDA&) const
{
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::VisiblyPushdownNPDA&) const
{
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::OneTapeDTM&) const
{
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::DFTA&) const
{
	throw exception::AlibException("Unsupported automaton type DFTA");
}

void AutomatonIterationEpsilonTransition::Visit(void*, const automaton::NFTA&) const
{
	throw exception::AlibException("Unsupported automaton type NFTA");
}

const AutomatonIterationEpsilonTransition AutomatonIterationEpsilonTransition::AUTOMATON_ITERATION_EPSILON_TRANSITION;

} /* namespace transform */

} /* namespace automaton */
