/*
 * Compaction.h
 *
 *  Created on: 2. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef COMPACTION_H_
#define COMPACTION_H_

#include "automaton/Automaton.h"

namespace automaton {

namespace transform {

/**
 * Transforms FSM to CompactNFA
 */
class Compaction : public automaton::VisitableConstFSMBase {
public:
	Compaction() {}

	static automaton::CompactNFA convert( const automaton::DFA& automaton);
	static automaton::CompactNFA convert( const automaton::NFA& automaton);
	static automaton::CompactNFA convert( const automaton::CompactNFA& automaton);

	/**
	 * Transforms
	 * @param automaton FSM to transform
	 * @return CompactNFA equivalent to given automaton
	 */
	static automaton::Automaton convert( const automaton::Automaton & automaton );

private:
	using automaton::VisitableConstFSMBase::Visit;

	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const MultiInitialStateNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;

	static const Compaction COMPACTION;
};

} /* namespace transform */

} /* namespace automaton */

#endif /* COMPACTION_H_ */
