/*
 * ToAutomaton.h
 *
 * Created on: 1. 11. 2013
 * Author: Tomas Pecka
 */

#ifndef _GRAMMAR_TO_AUTOMATON_H_
#define _GRAMMAR_TO_AUTOMATON_H_

#include <grammar/Grammar.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>
#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>

#include <automaton/Automaton.h>
#include <automaton/FSM/NFA.h>
#include <automaton/PDA/NPDA.h>

namespace grammar {

namespace convert {

/**
 * Converts regular grammar to nondeterministic finite automaton.
 * Sources: Melichar 2.98 (RightRG -> NFA) and 2.102 (LeftRG -> NFA).
 */
class ToAutomaton : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	ToAutomaton() {}

	/**
	 * Performs conversion.
	 * @param grammar Regular grammar to convert.
	 * @return FSM equivalent to source grammar.
	 */
	static automaton::Automaton convert(const grammar::Grammar& grammar);

	static automaton::NFA convert(const grammar::LeftRG& grammar);
	static automaton::NFA convert(const grammar::RightRG& grammar);

	template <class T>
	static automaton::NPDA convert(const T& grammar);

private:
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const ToAutomaton TO_AUTOMATON;
};

} /* namespace convert */

} /* namespace grammar */

#endif /* _GRAMMAR_TO_AUTOMATON_H_ */
