/*
 * ToAutomatonBototmUp.cpp
 *
 * Created on: 20. 5. 2015
 * Author: Tomas Pecka
 */
#include "ToAutomatonBottomUp.h"

#include <alphabet/BottomOfTheStackSymbol.h>
#include <alphabet/LabeledSymbol.h>
#include <exception/AlibException.h>

namespace grammar {

namespace convert {

automaton::Automaton ToAutomatonBottomUp::convert(const grammar::Grammar& grammar)
{
	 automaton::Automaton* out = NULL;
	 grammar.getData().Accept((void*) &out, ToAutomatonBottomUp::TO_AUTOMATON_BOTTOMUP);
	 automaton::Automaton res = std::move(*out);
	 delete out;
	 return res;
}

template <class T>
automaton::NPDA ToAutomatonBottomUp::convert(const T& grammar)
{
	automaton::State q = automaton::State('q');
	automaton::State r = automaton::State('r');

	automaton::NPDA automaton(q, alphabet::Symbol(alphabet::BottomOfTheStackSymbol::BOTTOM_OF_THE_STACK));
	automaton.addState(r);
	automaton.addFinalState(r);

	automaton.setInputAlphabet(grammar.getTerminalAlphabet());

	for(const alphabet::Symbol& symbol : grammar.getNonterminalAlphabet())
		automaton.addStackSymbol(symbol);
	for(const alphabet::Symbol& symbol : grammar.getTerminalAlphabet())
		automaton.addStackSymbol(symbol);

	for(const alphabet::Symbol& symbol : grammar.getTerminalAlphabet())
		automaton.addTransition(automaton.getInitialState(), symbol, std::vector<alphabet::Symbol>{}, automaton.getInitialState(), std::vector<alphabet::Symbol>{symbol});

	//for(const std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>>& kv : grammar.getRules())
	for(const auto& kv : grammar.getRules())
		for(const std::vector<alphabet::Symbol>& rhs : kv.second)
			automaton.addTransition(automaton.getInitialState(), rhs, automaton.getInitialState(), std::vector<alphabet::Symbol>{kv.first});

	automaton.addTransition(automaton.getInitialState(), std::vector<alphabet::Symbol>{alphabet::Symbol(alphabet::BottomOfTheStackSymbol::BOTTOM_OF_THE_STACK), grammar.getInitialSymbol()}, r, std::vector<alphabet::Symbol>{});

	return automaton;
}


void ToAutomatonBottomUp::Visit(void*, const grammar::RightRG&) const
{
	throw exception::AlibException("Unsupported grammar type RightRG");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::LeftRG&) const
{
	throw exception::AlibException("Unsupported grammar type LeftRG");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::RightLG&) const
{
	throw exception::AlibException("Unsupported grammar type RightLG");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::LeftLG&) const
{
	throw exception::AlibException("Unsupported grammar type LeftLG");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::LG& ) const {
	throw exception::AlibException("Unsupported grammar type LG");
}

void ToAutomatonBottomUp::Visit(void* userData, const grammar::CFG& grammar) const {
	automaton::Automaton* & out = *((automaton::Automaton**) userData);
	out = new automaton::Automaton(this->convert(grammar));
}

void ToAutomatonBottomUp::Visit(void* userData, const grammar::EpsilonFreeCFG& grammar) const {
	automaton::Automaton* & out = *((automaton::Automaton**) userData);
	out = new automaton::Automaton(this->convert(grammar));
}

void ToAutomatonBottomUp::Visit(void*, const grammar::CNF& ) const {
	throw exception::AlibException("Unsupported grammar type CFG");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::GNF& ) const {
	throw exception::AlibException("Unsupported grammar type GNF");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void ToAutomatonBottomUp::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const ToAutomatonBottomUp ToAutomatonBottomUp::TO_AUTOMATON_BOTTOMUP;

} /* namespace convert */

} /* namespace grammar */
