/*
 * GenerateUpToLength.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#include "GenerateUpToLength.h"

#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightRG.h>

#include <deque>

namespace grammar {

namespace generate {

template<class T>
std::set<string::LinearString> GenerateUpToLength::generate( const T & grammar, unsigned length ) {
	std::set<string::LinearString> res;

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> rules = grammar.getRawRules();
	if(grammar.getGeneratesEpsilon()) {
		res.insert(string::LinearString {});
		rules[grammar.getInitialSymbol()].erase(std::vector<alphabet::Symbol> {});
	}

	std::deque<std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol>>> data;
	data.push_back(std::make_pair(std::vector<alphabet::Symbol>{}, std::vector<alphabet::Symbol>{grammar.getInitialSymbol()}));
	while(!data.empty()) {
		std::pair<std::vector<alphabet::Symbol>, std::vector<alphabet::Symbol>> item = std::move(data.back());
		data.pop_back();

		alphabet::Symbol nonterminal = std::move(item.second.back());
		item.second.pop_back();
		if(rules.find(nonterminal) == rules.end()) continue;

		for(const std::vector<alphabet::Symbol>& rhs : rules.find(nonterminal)->second) {
			if(item.first.size() + item.second.size() + rhs.size() > length) continue;

			std::vector<alphabet::Symbol> newStack(item.second);
			std::vector<alphabet::Symbol> newString(item.first);
			newStack.insert(newStack.end(), rhs.rbegin(), rhs.rend());

			while(!newStack.empty() && grammar.getTerminalAlphabet().count(newStack.back())) {
				newString.push_back(std::move(newStack.back()));
				newStack.pop_back();
			}

			if(newStack.empty()) {
				res.insert(string::LinearString(newString));
			} else {
				data.push_back(std::make_pair(std::move(newString), std::move(newStack)));
			}
		}
	}
	return res;
}

template std::set<string::LinearString> GenerateUpToLength::generate( const grammar::EpsilonFreeCFG & grammar, unsigned length );
template std::set<string::LinearString> GenerateUpToLength::generate( const grammar::GNF & grammar, unsigned length );
template std::set<string::LinearString> GenerateUpToLength::generate( const grammar::CNF & grammar, unsigned length );
template std::set<string::LinearString> GenerateUpToLength::generate( const grammar::LeftRG & grammar, unsigned length );
template std::set<string::LinearString> GenerateUpToLength::generate( const grammar::RightRG & grammar, unsigned length );

std::set<string::LinearString> GenerateUpToLength::generate(const grammar::Grammar& grammar, unsigned length) {
	std::pair<std::set<string::LinearString>, unsigned> out;
	out.second = length;
	grammar.getData().Accept((void*) &out, GenerateUpToLength::GENERATE_UP_TO_LENGTH);
	return std::move(out.first);
}

void GenerateUpToLength::Visit(void*, const grammar::LeftLG&) const {
	throw exception::AlibException("Unsupported grammar type LeftLG");
}

void GenerateUpToLength::Visit(void* data, const grammar::LeftRG& grammar) const {
	std::pair<std::set<string::LinearString>, unsigned>& out = *((std::pair<std::set<string::LinearString>, unsigned>*) data);
	out.first = this->generate(grammar, out.second);
}

void GenerateUpToLength::Visit(void*, const grammar::RightLG&) const {
	throw exception::AlibException("Unsupported grammar type RightLG");
}

void GenerateUpToLength::Visit(void* data, const grammar::RightRG& grammar) const {
	std::pair<std::set<string::LinearString>, unsigned>& out = *((std::pair<std::set<string::LinearString>, unsigned>*) data);
	out.first = this->generate(grammar, out.second);
}

void GenerateUpToLength::Visit(void*, const grammar::LG&) const {
	throw exception::AlibException("Unsupported grammar type LG");
}

void GenerateUpToLength::Visit(void*, const grammar::CFG&) const {
	throw exception::AlibException("Unsupported grammar type CFG");
}

void GenerateUpToLength::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	std::pair<std::set<string::LinearString>, unsigned>& out = *((std::pair<std::set<string::LinearString>, unsigned>*) data);
	out.first = this->generate(grammar, out.second);
}

void GenerateUpToLength::Visit(void* data, const grammar::CNF& grammar) const {
	std::pair<std::set<string::LinearString>, unsigned>& out = *((std::pair<std::set<string::LinearString>, unsigned>*) data);
	out.first = this->generate(grammar, out.second);
}

void GenerateUpToLength::Visit(void* data, const grammar::GNF& grammar) const {
	std::pair<std::set<string::LinearString>, unsigned>& out = *((std::pair<std::set<string::LinearString>, unsigned>*) data);
	out.first = this->generate(grammar, out.second);
}

void GenerateUpToLength::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void GenerateUpToLength::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void GenerateUpToLength::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void GenerateUpToLength::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const GenerateUpToLength GenerateUpToLength::GENERATE_UP_TO_LENGTH;

} /* namespace generate */

} /* namespace grammar */

