/*
 * GenerateUpToLength.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef GRAMMAR_GENERATE_UP_TO_LENGTH_H_
#define GRAMMAR_GENERATE_UP_TO_LENGTH_H_

#include <grammar/Grammar.h>
#include <string/LinearString.h>
#include <set>

namespace grammar {

namespace generate {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class GenerateUpToLength : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	GenerateUpToLength() {}

	static std::set<string::LinearString> generate( const grammar::Grammar & grammar, unsigned length );

	template<class T>
	static std::set<string::LinearString> generate( const T & grammar, unsigned length );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const GenerateUpToLength GENERATE_UP_TO_LENGTH;
};

} /* namespace generate */

} /* namespace grammar */

#endif /* GRAMMAR_GENERATE_UP_TO_LENGTH_H_ */
