/*
 * IsLanguageEmpty.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "IsLanguageEmpty.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include "../properties/ProductiveNonterminals.h"

namespace grammar {

namespace properties {

template<class T>
bool IsLanguageEmpty::isLanguageEmpty( const T & grammar ) {
	return grammar::properties::ProductiveNonterminals::getProductiveNonterminals( grammar ).count( grammar.getInitialSymbol( ) );
}

template bool IsLanguageEmpty::isLanguageEmpty( const grammar::CFG & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::EpsilonFreeCFG & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::GNF & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::CNF & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::LG & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::LeftLG & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::LeftRG & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::RightLG & grammar );
template bool IsLanguageEmpty::isLanguageEmpty( const grammar::RightRG & grammar );

bool IsLanguageEmpty::isLanguageEmpty(const grammar::Grammar& grammar) {
	bool out;
	grammar.getData().Accept((void*) &out, IsLanguageEmpty::IS_LANGUAGE_EMPTY);
	return out;
}

void IsLanguageEmpty::Visit(void* data, const grammar::LeftLG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::LeftRG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::RightLG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::RightRG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::LG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::CFG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::CNF& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void* data, const grammar::GNF& grammar) const {
	bool & out = *((bool*) data);
	out = this->isLanguageEmpty(grammar);
}

void IsLanguageEmpty::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void IsLanguageEmpty::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void IsLanguageEmpty::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void IsLanguageEmpty::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const IsLanguageEmpty IsLanguageEmpty::IS_LANGUAGE_EMPTY;

} /* namespace properties */

} /* namespace grammar */

