/*
 * IsLanguageEmpty.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef IS_LANGUAGE_EMPTY_H_
#define IS_LANGUAGE_EMPTY_H_

#include <grammar/Grammar.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class IsLanguageEmpty : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	IsLanguageEmpty() {}

	static bool isLanguageEmpty( const grammar::Grammar & grammar );

	/*
	 * Melichar 3.6 - decides whether L( grammar ) = \0
	 *
	 * Severals steps implemented in method CFGTransformations::getProductiveNonTerminals();
	 * @see getProductiveNonTerminals
	 */
	template<class T>
	static bool isLanguageEmpty( const T & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const IsLanguageEmpty IS_LANGUAGE_EMPTY;
};

} /* namespace properties */

} /* namespace grammar */

#endif /* IS_LANGUAGE_EMPTY_H_ */
