/*
 * IsLanguageGeneratingEpsilon.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#include "IsLanguageGeneratingEpsilon.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include "../properties/NullableNonterminals.h"

namespace grammar {

namespace properties {

template<class T>
bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const T & grammar ) {
	return grammar::properties::NullableNonterminals::getNullableNonterminals( grammar ).count( grammar.getInitialSymbol( ) );
}

template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::CFG & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::EpsilonFreeCFG & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::GNF & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::CNF & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::LG & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::LeftLG & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::LeftRG & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::RightLG & grammar );
template bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon( const grammar::RightRG & grammar );

bool IsLanguageGeneratingEpsilon::isLanguageGeneratingEpsilon(const grammar::Grammar& grammar) {
	bool out;
	grammar.getData().Accept((void*) &out, IsLanguageGeneratingEpsilon::IS_LANGUAGE_GENERATING_EPSILON);
	return out;
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::LeftLG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::LeftRG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::RightLG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::RightRG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::LG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::CFG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::CNF& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void* data, const grammar::GNF& grammar) const {
	bool & out = *((bool*) data);
	out = std::move(this->isLanguageGeneratingEpsilon(grammar));
}

void IsLanguageGeneratingEpsilon::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void IsLanguageGeneratingEpsilon::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void IsLanguageGeneratingEpsilon::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void IsLanguageGeneratingEpsilon::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const IsLanguageGeneratingEpsilon IsLanguageGeneratingEpsilon::IS_LANGUAGE_GENERATING_EPSILON;

} /* namespace properties */

} /* namespace grammar */

