/*
 * IsLanguageGeneratingEpsilon.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Jan Travnicek
 */

#ifndef IS_LANGUAGE_GENERATING_EPSILON_H_
#define IS_LANGUAGE_GENERATING_EPSILON_H_

#include <grammar/Grammar.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class IsLanguageGeneratingEpsilon : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	IsLanguageGeneratingEpsilon() {}

	static bool isLanguageGeneratingEpsilon( const grammar::Grammar & grammar );

	/*
	 * Melichar 3.6 - decides whether L( grammar ) = \0
	 *
	 * Severals steps implemented in method CFGTransformations::getProductiveNonTerminals();
	 * @see getProductiveNonTerminals
	 */
	template<class T>
	static bool isLanguageGeneratingEpsilon( const T & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const IsLanguageGeneratingEpsilon IS_LANGUAGE_GENERATING_EPSILON;
};

} /* namespace properties */

} /* namespace grammar */

#endif /* IS_LANGUAGE_GENERATING_EPSILON_H_ */
