/*
 * NullableNonterminals.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "NullableNonterminals.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <set>
#include <deque>
#include <algorithm>

namespace grammar {

namespace properties {

template<class T>
std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals(const T& grammar) {
	std::deque<std::set<alphabet::Symbol>> Ni;

	Ni.push_back(std::set<alphabet::Symbol>{ });
	int i = 1;

	while(true) {
		Ni.push_back(std::set<alphabet::Symbol>{ });
		for(const auto& rule : grammar.getRawRules()) {
			for(const auto& rhs : rule.second) {
				if(rhs.size() == 0 || std::all_of(rhs.begin(), rhs.end(), [Ni, i](const alphabet::Symbol& symb){return Ni.at(i-1).count(symb);})) {
					Ni.at(i).insert(rule.first);
				}
			}
		}

		if(Ni.at(i) == Ni.at(i-1))
			break;

		i += 1;
	}

	return Ni.at(i);
}

template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::CFG& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::EpsilonFreeCFG& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::GNF& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::CNF& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::LG& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::LeftLG& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::LeftRG& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::RightLG& grammar );
template std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals( const grammar::RightRG& grammar );

std::set<alphabet::Symbol> NullableNonterminals::getNullableNonterminals(const grammar::Grammar& grammar) {
	std::set<alphabet::Symbol> out;
	grammar.getData().Accept((void*) &out, NullableNonterminals::NULLABLE_NONTERMINALS);
	return out;
}

void NullableNonterminals::Visit(void* data, const grammar::LeftLG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::LeftRG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::RightLG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::RightRG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::LG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::CFG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::CNF& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void* data, const grammar::GNF& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getNullableNonterminals(grammar));
}

void NullableNonterminals::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void NullableNonterminals::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void NullableNonterminals::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void NullableNonterminals::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const NullableNonterminals NullableNonterminals::NULLABLE_NONTERMINALS;

} /* namespace properties */

} /* namespace grammar */
