/*
 * NullableNonterminals.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef NULLABLE_NONTERMINALS_H_
#define NULLABLE_NONTERMINALS_H_

#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class NullableNonterminals : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	NullableNonterminals() {}

	static std::set<alphabet::Symbol> getNullableNonterminals( const grammar::Grammar & grammar );

	/**
	 * Retrieve all nullable nonterminals from grammar
	 * Nullable nonterminal is such nonterminal A for which holds that A ->^* \eps
	 *
	 * Source: Melichar, algorithm 2.4, step 1
	 *
	 * @param grammar grammar
	 * @return set of nullable nonterminals from grammar
	 */
	template<class T>
	static std::set<alphabet::Symbol> getNullableNonterminals(const T& grammar);

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const NullableNonterminals NULLABLE_NONTERMINALS;
};

} /* namespace properties */

} /* namespace grammar */

#endif /* NULLABLE_NONTERMINALS_H_ */
