/*
 * ProductiveNonterminals.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "ProductiveNonterminals.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <set>
#include <deque>
#include <algorithm>

namespace grammar {

namespace properties {

template<class T>
std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const T & grammar ) {
	// 1.
	std::deque<std::set<alphabet::Symbol>> Ni;
	Ni.push_back( std::set<alphabet::Symbol>( ) );

	int i = 1;

	// 2.
	while( true ) {
		Ni.push_back( Ni.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			for( const auto & rhs : rule.second ) {
				if( std::all_of( rhs.begin( ), rhs.end( ), [ i, Ni, grammar ]( const alphabet::Symbol & symbol ) -> bool {
					return Ni.at( i - 1 ) . count( symbol ) || grammar.getTerminalAlphabet( ). count( symbol );
				} ) )
					Ni.at( i ).insert( rule.first );
			}
		}

		if( Ni.at( i ) == Ni.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Ni.at( i );
}

template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::CFG & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::EpsilonFreeCFG & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::GNF & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::CNF & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::LG & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::LeftLG & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::LeftRG & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::RightLG & grammar );
template std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals( const grammar::RightRG & grammar );

std::set<alphabet::Symbol> ProductiveNonterminals::getProductiveNonterminals(const grammar::Grammar& grammar) {
	std::set<alphabet::Symbol> out;
	grammar.getData().Accept((void*) &out, ProductiveNonterminals::PRODUCTIVE_NONTERMINALS);
	return out;
}

void ProductiveNonterminals::Visit(void* data, const grammar::LeftLG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::LeftRG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::RightLG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::RightRG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::LG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::CFG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::CNF& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void* data, const grammar::GNF& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getProductiveNonterminals(grammar));
}

void ProductiveNonterminals::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void ProductiveNonterminals::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void ProductiveNonterminals::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void ProductiveNonterminals::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const ProductiveNonterminals ProductiveNonterminals::PRODUCTIVE_NONTERMINALS;

} /* namespace properties */

} /* namespace grammar */
