/*
 * GrammarPropertiesCFG.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef PRODUCTIVE_NONTERMINALS_H_
#define PRODUCTIVE_NONTERMINALS_H_

#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class ProductiveNonterminals : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	ProductiveNonterminals() {}

	static std::set<alphabet::Symbol> getProductiveNonterminals( const grammar::Grammar & grammar );

	/**
	 * Implements steps 1 through 3 in Melichar 3.6
	 */
	template<class T>
	static std::set<alphabet::Symbol> getProductiveNonterminals( const T & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const ProductiveNonterminals PRODUCTIVE_NONTERMINALS;
};

} /* namespace properties */

} /* namespace grammar */

#endif /* PRODUCTIVE_NONTERMINALS_H_ */
