/*
 * UnreachableSymbols.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "UnreachableSymbols.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include <algorithm>
#include <deque>
#include <set>

namespace grammar {

namespace properties {

template<class T>
std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const T & grammar ) {
	// 1
	std::deque<std::set<alphabet::Symbol>> Vi;
	Vi.push_back( std::set<alphabet::Symbol>( ) );
	Vi.at( 0 ).insert( grammar.getInitialSymbol( ) );

	int i = 1;

	// 2.
	while( true ) {
		Vi.push_back( Vi.at( i - 1 ) );

		for( const auto & rule : grammar.getRawRules( ) ) {
			if( Vi.at( i - 1 ).count( rule.first ) ) {
				for( const auto & rhs : rule.second ) {
					Vi.at( i ).insert( rhs.begin( ), rhs.end( ) );
				}
			}
		}


		if( Vi.at( i ) == Vi.at( i - 1 ) )
			break;

		i = i + 1;
	}

	// 3.
	return Vi.at( i );
}

template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::CFG & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::EpsilonFreeCFG & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::GNF & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::CNF & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::LG & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::LeftLG & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::LeftRG & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::RightLG & grammar );
template std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols( const grammar::RightRG & grammar );

std::set<alphabet::Symbol> UnreachableSymbols::getUnreachableSymbols(const grammar::Grammar& grammar) {
	std::set<alphabet::Symbol> out;
	grammar.getData().Accept((void*) &out, UnreachableSymbols::UNREACHABLE_NONTERMINALS);
	return out;
}

void UnreachableSymbols::Visit(void* data, const grammar::LeftLG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::LeftRG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::RightLG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::RightRG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::LG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::CFG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::CNF& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void* data, const grammar::GNF& grammar) const {
	std::set<alphabet::Symbol> & out = *((std::set<alphabet::Symbol>*) data);
	out = std::move(this->getUnreachableSymbols(grammar));
}

void UnreachableSymbols::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void UnreachableSymbols::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void UnreachableSymbols::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void UnreachableSymbols::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const UnreachableSymbols UnreachableSymbols::UNREACHABLE_NONTERMINALS;

} /* namespace properties */

} /* namespace grammar */
