/*
 * UnreachableSymbols.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef UNREACHABLE_SYMBOLS_H_
#define UNREACHABLE_SYMBOLS_H_

#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace properties {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class UnreachableSymbols : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	UnreachableSymbols() {}

	static std::set<alphabet::Symbol> getUnreachableSymbols( const grammar::Grammar & grammar );

	/**
	 * Implements
	 */
	template<class T>
	static std::set<alphabet::Symbol> getUnreachableSymbols( const T & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const UnreachableSymbols UNREACHABLE_NONTERMINALS;
};

} /* namespace properties */

} /* namespace grammar */

#endif /* UNREACHABLE_SYMBOLS_H_ */
