/*
 * Trim.cpp
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "Trim.h"

#include <grammar/ContextFree/CFG.h>
#include <grammar/ContextFree/EpsilonFreeCFG.h>
#include <grammar/ContextFree/GNF.h>
#include <grammar/ContextFree/CNF.h>
#include <grammar/ContextFree/LG.h>
#include <grammar/Regular/LeftLG.h>
#include <grammar/Regular/LeftRG.h>
#include <grammar/Regular/RightLG.h>
#include <grammar/Regular/RightRG.h>

#include "UnreachableSymbolsRemover.h"
#include "UnproductiveSymbolsRemover.h"

namespace grammar {

namespace simplify {

template<class T>
T Trim::trim( const T & grammar ) {
	return grammar::simplify::UnreachableSymbolsRemover::remove( grammar::simplify::UnproductiveSymbolsRemover::remove( grammar ) );
}

template grammar::CFG Trim::trim( const grammar::CFG & grammar );
template grammar::EpsilonFreeCFG Trim::trim( const grammar::EpsilonFreeCFG & grammar );
template grammar::GNF Trim::trim( const grammar::GNF & grammar );
template grammar::CNF Trim::trim( const grammar::CNF & grammar );
template grammar::LG Trim::trim( const grammar::LG & grammar );
template grammar::LeftLG Trim::trim( const grammar::LeftLG & grammar );
template grammar::LeftRG Trim::trim( const grammar::LeftRG & grammar );
template grammar::RightLG Trim::trim( const grammar::RightLG & grammar );
template grammar::RightRG Trim::trim( const grammar::RightRG & grammar );

grammar::Grammar Trim::trim(const grammar::Grammar& grammar) {
	grammar::Grammar* out = NULL;
	grammar.getData().Accept((void*) &out, Trim::TRIM);
	grammar::Grammar res = std::move(*out);
	delete out;
	return res;
}

void Trim::Visit(void* data, const grammar::LeftLG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::LeftRG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::RightLG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::RightRG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::LG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::CFG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::EpsilonFreeCFG& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::CNF& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void* data, const grammar::GNF& grammar) const {
	grammar::Grammar* & out = *((grammar::Grammar**) data);
	out = new grammar::Grammar(this->trim(grammar));
}

void Trim::Visit(void*, const grammar::CSG&) const {
	throw exception::AlibException("Unsupported grammar type CSG");
}

void Trim::Visit(void*, const grammar::NonContractingGrammar&) const {
	throw exception::AlibException("Unsupported grammar type NonConctractingGrammar");
}

void Trim::Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type ContextPreservingUnrestrictedGrammar");
}

void Trim::Visit(void*, const grammar::UnrestrictedGrammar&) const {
	throw exception::AlibException("Unsupported grammar type UnrestrictedGrammar");
}

const Trim Trim::TRIM;

} /* namespace simplify */

} /* namespace grammar */

