/*
 * Trim.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef GRAMMAR_TRIM_H_
#define GRAMMAR_TRIM_H_

#include <grammar/Grammar.h>

namespace grammar {

namespace simplify {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class Trim : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	Trim() {}

	static grammar::Grammar trim( const grammar::Grammar & grammar );

	/**
	 * Removes unproductive and useless symbols - Melichar 3.12
	 */
	template<class T>
	static T trim( const T & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const Trim TRIM;
};

} /* namespace simplify */

} /* namespace grammar */

#endif /* GRAMMAR_TRIM_H_ */
