/*
 * UnreachableSymbolsRemover.h
 *
 *  Created on: 22. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef UNREACHABLE_SYMBOLS_REMOVER_H_
#define UNREACHABLE_SYMBOLS_REMOVER_H_

#include <algorithm>
#include <deque>
#include <set>

#include <exception/AlibException.h>
#include <grammar/Grammar.h>
#include <alphabet/Symbol.h>

namespace grammar {

namespace simplify {

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class UnreachableSymbolsRemover : public grammar::VisitableGrammarBase::const_visitor_type {
public:
	UnreachableSymbolsRemover() {}

	static grammar::Grammar remove( const grammar::Grammar & automaton );

	/*
	 * Removes unreachable symbols - Melichar 3.9
	 */
	template<class T>
	static T remove( const T & grammar );

private:
	void Visit(void*, const grammar::LeftLG& grammar) const;
	void Visit(void*, const grammar::LeftRG& grammar) const;
	void Visit(void*, const grammar::RightLG& grammar) const;
	void Visit(void*, const grammar::RightRG& grammar) const;
	void Visit(void*, const grammar::LG& grammar) const;
	void Visit(void*, const grammar::CFG& grammar) const;
	void Visit(void*, const grammar::EpsilonFreeCFG& grammar) const;
	void Visit(void*, const grammar::CNF& grammar) const;
	void Visit(void*, const grammar::GNF& grammar) const;
	void Visit(void*, const grammar::CSG& grammar) const;
	void Visit(void*, const grammar::NonContractingGrammar& grammar) const;
	void Visit(void*, const grammar::ContextPreservingUnrestrictedGrammar& grammar) const;
	void Visit(void*, const grammar::UnrestrictedGrammar& grammar) const;

	static const UnreachableSymbolsRemover UNREACHABLE_SYMBOLS_REMOVER;
};

} /* namespace simplify */

} /* namespace grammar */

#endif /* UNREACHABLE_SYMBOLS_REMOVER_H_ */
