#ifndef GRAPH_FLOYD_WARSHALL_FORD_H_
#define GRAPH_FLOYD_WARSHALL_FORD_H_

#include <unordered_map>

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph
{

namespace shortestpath
{

// Floyd-Warshall only works on graphs without negative-weight cycle
// However it will detect the negative-weight cycle and throw exception
//
// note: negative-weight undirected edge = negative-weight cycle
// note: std::numeric_limits<int>::max() is used as infinity

class FloydWarshall : public graph::VisitableGraphBase::const_visitor_type
{
public:
	FloydWarshall() {}

	typedef std::unordered_map<Node, std::unordered_map<Node, int>> Result;

	static Result floydwarshall(const Graph &graph);

	static Result floydwarshall(const DirectedGraph &graph);
	static Result floydwarshall(const UndirectedGraph &graph);

private:
	void Visit(void *data, const DirectedGraph &graph) const;
	void Visit(void *data, const UndirectedGraph &graph) const;

	static const FloydWarshall FLOYD_WARSHALL;
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_FLOYD_WARSHALL_FORD_H_
