#include "TopologicalSort.h"
#include "../traverse/Dfs.h"

#include <exception/AlibException.h>

namespace graph
{

namespace sort
{

static TopologicalSort::Result topsort_impl(const DirectedGraph &graph)
{
	TopologicalSort::Result s;
	std::unordered_map<Node, bool> visited;

	for (const Node &n : graph.getNodes()) {
		traverse::Dfs::dfs(graph, n, [&s, &visited](const Node &n, const Node&, int, int) {
			if (visited.find(n) != visited.end()) {
				return false;
			}
			visited.insert({n, true});
			s.push_front(n);
			return true;
		});
	}

	return s;
}

TopologicalSort::Result TopologicalSort::topologicalsort(const Graph &graph)
{
	Result res;
	graph.getData().Accept(static_cast<void*>(&res), TOPOLOGICAL_SORT);
	return res;
}

TopologicalSort::Result TopologicalSort::topologicalsort(const DirectedGraph &graph)
{
	return topsort_impl(graph);
}

void TopologicalSort::Visit(void *data, const DirectedGraph &graph) const
{
	Result *r = static_cast<Result*>(data);
	*r = topologicalsort(graph);
}

void TopologicalSort::Visit(void *data, const UndirectedGraph &graph) const
{
	(void) data;
	(void) graph;
	throw exception::AlibException("Unsupported graph type UndirectedGraph");
}

const TopologicalSort TopologicalSort::TOPOLOGICAL_SORT;

} // namespace sort

} // namespace graph
