#include "Bfs.h"

#include <queue>
#include <unordered_map>

namespace graph
{

namespace traverse
{

struct Data
{
	Node start;
	Bfs::Function func;
	Bfs::FunctionExt func2;
};

template <typename T>
static void bfs_impl(const T &graph, const Node &start, Bfs::Function func)
{
	std::unordered_map<Node, bool> visited;
	std::queue<Node> q;

	q.push(start);
	visited[start] = true;

	while (!q.empty()) {
		Node n = q.front(); q.pop();

		if (!func(n)) {
			return;
		}

		for (const Node &e : graph.neighbors(n)) {
			if (visited.find(e) == visited.end()) {
				visited[e] = true;
				q.push(e);
			}
		}
	}
}

template <typename T>
static void bfs2_impl(const T &graph, const Node &start, Bfs::FunctionExt func)
{
	std::unordered_map<Node, int> distances;
	std::unordered_map<Node, Node> predecessors;
	std::queue<Node> q;

	q.push(start);
	distances[start] = 0;
	predecessors.insert(std::make_pair(start, start));

	while (!q.empty()) {
		Node n = q.front(); q.pop();

		if (!func(n, predecessors.find(n)->second, distances.find(n)->second)) {
			return;
		}

		for (const Node &e : graph.neighbors(n)) {
			if (distances.find(e) == distances.end()) {
				distances[e] = distances.at(n) + 1;
				predecessors.insert(std::make_pair(e, n));
				q.push(e);
			}
		}
	}
}

void Bfs::bfs(const Graph &graph, const Node &start, Bfs::Function func)
{
	Data data = { start, func, nullptr };
	graph.getData().Accept(static_cast<void*>(&data), BFS);
}

void Bfs::bfs(const Graph &graph, const Node &start, Bfs::FunctionExt func)
{
	Data data = { start, nullptr, func };
	graph.getData().Accept(static_cast<void*>(&data), BFS);
}

void Bfs::bfs(const DirectedGraph &graph, const Node &start, Bfs::Function func)
{
	bfs_impl(graph, start, func);
}

void Bfs::bfs(const DirectedGraph &graph, const Node &start, Bfs::FunctionExt func)
{
	bfs2_impl(graph, start, func);
}

void Bfs::bfs(const UndirectedGraph &graph, const Node &start, Bfs::Function func)
{
	bfs_impl(graph, start, func);
}

void Bfs::bfs(const UndirectedGraph &graph, const Node &start, Bfs::FunctionExt func)
{
	bfs2_impl(graph, start, func);
}

void Bfs::Visit(void *data, const DirectedGraph &graph) const
{
	Data d = *static_cast<Data*>(data);
	if (d.func) {
		bfs(graph, d.start, d.func);
	} else if (d.func2) {
		bfs(graph, d.start, d.func2);
	}
}

void Bfs::Visit(void *data, const UndirectedGraph &graph) const
{
	Data d = *static_cast<Data*>(data);
	if (d.func) {
		bfs(graph, d.start, d.func);
	} else if (d.func2) {
		bfs(graph, d.start, d.func2);
	}
}

const Bfs Bfs::BFS;

} // namespace traverse

} // namespace graph
