#ifndef GRAPH_BFS_H_
#define GRAPH_BFS_H_

#include <graph/Graph.h>
#include <graph/directed/DirectedGraph.h>
#include <graph/undirected/UndirectedGraph.h>

namespace graph
{

namespace traverse
{

// func is called for each visited node, traversal stops when returning false
//
// bool Function(const Node &visitedNode);
// bool FunctionExt(const Node &visitedNode, const Node &predecessor, int distanceFromStart);

class Bfs : public graph::VisitableGraphBase::const_visitor_type
{
public:
	Bfs() {}

	typedef std::function<bool(const Node&)> Function;
	typedef std::function<bool(const Node&, const Node&, int)> FunctionExt;

	static void bfs(const Graph &graph, const Node &start, Function func);
	static void bfs(const Graph &graph, const Node &start, FunctionExt func);

	static void bfs(const DirectedGraph &graph, const Node &start, Function func);
	static void bfs(const DirectedGraph &graph, const Node &start, FunctionExt func);
	static void bfs(const UndirectedGraph &graph, const Node &start, Function func);
	static void bfs(const UndirectedGraph &graph, const Node &start, FunctionExt func);

private:
	void Visit(void *data, const DirectedGraph &graph) const;
	void Visit(void *data, const UndirectedGraph &graph) const;

	static const Bfs BFS;
};

} // namespace traverse

} // namespace graph

#endif // GRAPH_BFS_H_
