/*
 * ToAutomatonDerivation.h
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TO_AUTOMATON_DERIVATION_H_
#define TO_AUTOMATON_DERIVATION_H_

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

#include <automaton/Automaton.h>
#include <automaton/FSM/NFA.h>

namespace regexp {

namespace convert {

/**
 * Converts regular expression to finite automaton using BrzozowskiDerivation algorithm (derivations of regular expressions).
 * Source: Melichar 2.110
 */
class ToAutomatonDerivation : public regexp::VisitableRegExpBase::const_visitor_type
{
public:
	ToAutomatonDerivation() {}

	/**
	 * Performs conversion.
	 * @return FSM equivalent to original regular expression.
	 */
	static automaton::Automaton convert(const regexp::RegExp& regexp);

	template<class T>
	static automaton::NFA convert(const T& regexp);

private:
	void Visit(void* , const regexp::FormalRegExp& regexp) const;
	void Visit(void* , const regexp::UnboundedRegExp& regexp) const;

	static const ToAutomatonDerivation TO_AUTOMATON_DERIVATION;
};

} /* namespace convert */

} /* namespace regexp */

#endif /* TO_AUTOMATON_DERIVATION_H_ */
