/*
 * ToAutomatonThompson.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */
#include "ToAutomatonThompson.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"
#include "regexp/formal/FormalRegExpElements.h"
#include <tuple>

namespace regexp {

namespace convert {

automaton::Automaton ToAutomatonThompson::convert(const regexp::RegExp& regexp)
{
	automaton::Automaton* out = NULL;
	regexp.getData().Accept((void*) &out, ToAutomatonThompson::TO_AUTOMATON_THOMPSON);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

template<class T>
automaton::EpsilonNFA ToAutomatonThompson::convert(const T& regexp)
{	//FIXME use actual algorithms that implement product alternation and iteration of re over automata and remove terrible TERRIBLE hack with dummy initial state
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> out(automaton::EpsilonNFA(automaton::State(0)), 1, nullptr, nullptr);

	automaton::EpsilonNFA& automaton = std::get<0>(out);
	automaton.setInputAlphabet(regexp.getAlphabet());

	regexp.getRegExp().Accept((void*) &out, ToAutomatonThompson::TO_AUTOMATON_THOMPSON);

	automaton.setInitialState(*std::get<2>(out));
	automaton.setFinalStates(std::set<automaton::State>{*std::get<3>(out)});

	automaton.removeState(automaton::State(0));

	return automaton;
}

// ----------------------------------------------------------------------------

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExp& regexp) const
{
	automaton::Automaton* & out = *((automaton::Automaton**) userData);
	out = new automaton::Automaton(this->convert(regexp));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExp& regexp) const
{
	automaton::Automaton* & out = *((automaton::Automaton**) userData);
	out = new automaton::Automaton(this->convert(regexp));
}

// ----------------------------------------------------------------------------

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExpAlternation& alternation) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	static_cast<const regexp::FormalRegExpElement&>(alternation.getLeftElement()).Accept(userData, *this);
	automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
	automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);

	static_cast<const regexp::FormalRegExpElement&>(alternation.getRightElement()).Accept(userData, *this);
	automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
	automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);

	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExpConcatenation& concatenation) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

	automaton::EpsilonNFA& automaton = std::get<0>(out);

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getLeftElement()).Accept(userData, *this);
	const automaton::State* leftHead = std::get<2>(out);
	const automaton::State* leftTail = std::get<3>(out);

	static_cast<const regexp::FormalRegExpElement&>(concatenation.getRightElement()).Accept(userData, *this);
	automaton.addTransition(*leftTail, string::Epsilon::EPSILON, *std::get<2>(out));

	std::get<2>(out) = &(*automaton.getStates().find(*leftHead));
	// std::get<3>(out) = std::get<3>(out);
}

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExpIteration& iteration) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	static_cast<const regexp::FormalRegExpElement&>(iteration.getElement()).Accept(userData, *this);
	automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
	automaton.addTransition(head, string::Epsilon::EPSILON, tail);
	automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);
	automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, *std::get<2>(out));

	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExpSymbol& symbol) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	automaton.addTransition(head, symbol.getSymbol(), tail);
	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExpEpsilon&) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	automaton.addTransition(head, string::Epsilon::EPSILON, tail);
	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::FormalRegExpEmpty&) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;

	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);

	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

// ----------------------------------------------------------------------------

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExpAlternation& alternation) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	for(const auto& element : alternation.getElements())
	{
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(userData, *this);
		automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
		automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);
	}

	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExpConcatenation& concatenation) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	std::vector<std::pair<const automaton::State*, const automaton::State*>> tails;
	for(const auto& element : concatenation.getElements())
	{
		static_cast<const regexp::UnboundedRegExpElement&>(*element).Accept(userData, *this);
		tails.push_back(std::make_pair(std::get<2>(out), std::get<3>(out)));
	}

	for(size_t i = 1; i < tails.size(); i++)
		automaton.addTransition(*tails[i-1].second, string::Epsilon::EPSILON, *tails[i].first);

	std::get<2>(out) = tails[0].first;
	std::get<3>(out) = tails[tails.size()-1].second;
}

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExpIteration& iteration) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	static_cast<const regexp::UnboundedRegExpElement&>(iteration.getElement()).Accept(userData, *this);
	automaton.addTransition(head, string::Epsilon::EPSILON, *std::get<2>(out));
	automaton.addTransition(head, string::Epsilon::EPSILON, tail);
	automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, tail);
	automaton.addTransition(*std::get<3>(out), string::Epsilon::EPSILON, *std::get<2>(out));

	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExpSymbol& symbol) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	automaton.addTransition(head, symbol.getSymbol(), tail);
	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExpEpsilon&) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	automaton.addTransition(head, string::Epsilon::EPSILON, tail);
	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

void ToAutomatonThompson::Visit(void* userData, const regexp::UnboundedRegExpEmpty&) const
{
	std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*> &out = *(std::tuple<automaton::EpsilonNFA, int, const automaton::State*, const automaton::State*>*) userData;
	automaton::EpsilonNFA& automaton = std::get<0>(out);

	automaton::State head = automaton::State(std::get<1>(out)++);
	automaton::State tail = automaton::State(std::get<1>(out)++);
	automaton.addState(head);
	automaton.addState(tail);

	std::get<2>(out) = &(*automaton.getStates().find(head));
	std::get<3>(out) = &(*automaton.getStates().find(tail));
}

const ToAutomatonThompson ToAutomatonThompson::TO_AUTOMATON_THOMPSON;

} /* namespace convert */

} /* namespace regexp */
