/*
 * ToGrammarRightRGDerivation.h
 *
 *  Created on: 6. 3. 2014
 *	  Author: Tomas Pecka
 */

#ifndef TO_GRAMMAR_RIGHT_RG_DERIVATION_H_
#define TO_GRAMMAR_RIGHT_RG_DERIVATION_H_

#include <grammar/Regular/RightRG.h>
#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace regexp {

namespace convert {

/**
 * Converts reg. expression to right regular grammar using brzozowski derivation algorithm.
 * Source: Melichar 2.137
 */
class ToGrammarRightRGDerivation : public regexp::VisitableRegExpBase::const_visitor_type
{
public:
	ToGrammarRightRGDerivation() {}

	/**
	 * Performs conversion.
	 * @return right regular grammar equivalent to source regexp.
	 */
	static grammar::RightRG convert(const regexp::RegExp& regexp);

	template <class T>
	static grammar::RightRG convert(const T& regexp);

private:
	void Visit(void*, const regexp::FormalRegExp& regexp) const;
	void Visit(void*, const regexp::UnboundedRegExp& regexp) const;

	static const ToGrammarRightRGDerivation TO_GRAMMAR_RIGHT_RG_DERIVATION;
};

} /* namespace convert */

} /* namespace regexp */

#endif /* TO_GRAMMAR_RIGHT_RG_DERIVATION_H_ */
