/*
 * ToGrammarRightRGGlushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "ToGrammarRightRGGlushkov.h"

#include <algorithm>

#include <alphabet/LabeledSymbol.h>

#include <label/LabelPairLabel.h>
#include <label/ObjectLabel.h>

#include <object/Object.h>

#include "../GlushkovTraversal.h"
#include "../GlushkovPair.h"
#include "../GlushkovSymbol.h"

#include "../properties/RegExpEpsilon.h"

namespace regexp {

namespace convert {

grammar::Grammar ToGrammarRightRGGlushkov::convert(const regexp::RegExp& regexp)
{
	grammar::Grammar* out = NULL;
	regexp.getData().Accept((void*) &out, ToGrammarRightRGGlushkov::TO_GRAMMAR_RIGHT_RG_GLUSHKOV);
	grammar::Grammar res = std::move(*out);
	delete out;
	return res;
}

grammar::RightRG ToGrammarRightRGGlushkov::convert(const regexp::UnboundedRegExp& regexp)
{
	alphabet::Symbol S = alphabet::symbolFrom("S");
	grammar::RightRG grammar(S);

	// step 1
	grammar.setTerminalAlphabet(regexp.getAlphabet());

	// steps 2, 3, 4
	std::set<regexp::GlushkovPair> pairs;
	const std::set<regexp::GlushkovSymbol> first = regexp::GlushkovTraversal::first(regexp);
	const std::set<regexp::GlushkovSymbol> last = regexp::GlushkovTraversal::last(regexp);
	for(const auto& x : regexp::GlushkovTraversal::getSymbols(regexp))
		for(const auto& f : regexp::GlushkovTraversal::follow(regexp, x))
			pairs.insert(regexp::GlushkovPair(x, f));
	// \e in q0 check is in step 7

	// step 5
	std::map<regexp::GlushkovSymbol, alphabet::Symbol> symbolMap;

	for(const auto& symbol : regexp::GlushkovTraversal::getSymbols(regexp))
	{
		alphabet::Symbol nt(alphabet::LabeledSymbol(label::Label(label::LabelPairLabel(std::make_pair(label::Label( label::ObjectLabel( alib::Object( symbol.getInputSymbol( ).getData() ) ) ), label::labelFrom(symbol.getId()))))));
		symbolMap.insert(std::make_pair(symbol, nt));
		grammar.addNonterminalSymbol(nt);
	}

	// step 6
	for(const auto& symbol : first)
		grammar.addRule(S, std::make_pair(symbol.getInputSymbol(), symbolMap.find(symbol)->second));

	for(const auto& pair : pairs)
	{
		const alphabet::Symbol& a = symbolMap.find(pair.getFirst())->second;
		const alphabet::Symbol& b = symbolMap.find(pair.getSecond())->second;
		grammar.addRule(a, std::make_pair(pair.getSecond().getInputSymbol(), b));
	}

	// step 7
	for(const auto& symbol : last)
	{
		/*
		 * for all rules where ns.m_nonTerminal is on rightSide:
		 *  add Rule: leftSide -> symbol.getSymbol( )
		 *  unless it already exists
		 */

		const alphabet::Symbol& a = symbolMap.find(symbol)->second;

		for(const auto& rule : grammar.getRawRules())
			for(const auto& rhs : rule.second)
				if(std::find(rhs.begin(), rhs.end(), a) != rhs.end())
					grammar.addRule(rule.first, rhs.at(0));
	}

	if(regexp::properties::RegExpEpsilon::languageContainsEpsilon(regexp))
		grammar.setGeneratesEpsilon(true);

	return grammar;
}

grammar::RightRG ToGrammarRightRGGlushkov::convert(const regexp::FormalRegExp& /* regexp */) {
	throw exception::AlibException("Glushkov: Converting FormalRegExp NYI"); // TODO
}

void ToGrammarRightRGGlushkov::Visit(void* userData, const regexp::FormalRegExp& regexp) const
{
	grammar::Grammar* & out = *((grammar::Grammar**) userData);
	out = new grammar::Grammar(this->convert(regexp));
}

void ToGrammarRightRGGlushkov::Visit(void* userData, const regexp::UnboundedRegExp& regexp) const
{
	grammar::Grammar* & out = *((grammar::Grammar**) userData);
	out = new grammar::Grammar(this->convert(regexp));
}

const ToGrammarRightRGGlushkov ToGrammarRightRGGlushkov::TO_GRAMMAR_RIGHT_RG_GLUSHKOV;

} /* namespace convert */

} /* namespace regexp */
