/*
 * RegExpEmpty.h
 *
 *  Created on: 19. 1. 2014
 *	  Author: Tomas Pecka
 */

#ifndef REG_EXP_EMPTY_H_
#define REG_EXP_EMPTY_H_

#include <common/multipleDispatch.hpp>

#include <regexp/RegExp.h>
#include <regexp/formal/FormalRegExp.h>
#include <regexp/unbounded/UnboundedRegExp.h>

namespace regexp {

namespace properties {

/**
 * Determines whether regular expression is empty (regexp == \0)
 *
 */
class RegExpEmpty : public std::SingleDispatch<bool, regexp::RegExpBase>, regexp::FormalRegExpElement::const_visitor_type, regexp::UnboundedRegExpElement::const_visitor_type {
public:
	RegExpEmpty() {}

	static bool languageIsEmpty(const regexp::RegExp& regexp);

	static bool languageIsEmpty(const regexp::FormalRegExpElement& regexp);
	static bool languageIsEmpty(const regexp::FormalRegExp& regexp);

	static bool languageIsEmpty(const regexp::UnboundedRegExp& regexp);
	static bool languageIsEmpty(const regexp::UnboundedRegExpElement& regexp);

private:
	void Visit(void* data, const regexp::UnboundedRegExpAlternation& alternation) const;
	void Visit(void* data, const regexp::UnboundedRegExpConcatenation& concatenation) const;
	void Visit(void* data, const regexp::UnboundedRegExpIteration& iteration) const;
	void Visit(void* data, const regexp::UnboundedRegExpSymbol& symbol) const;
	void Visit(void* data, const regexp::UnboundedRegExpEmpty& empty) const;
	void Visit(void* data, const regexp::UnboundedRegExpEpsilon& epsilon) const;

	void Visit(void* data, const regexp::FormalRegExpAlternation& alternation) const;
	void Visit(void* data, const regexp::FormalRegExpConcatenation& concatenation) const;
	void Visit(void* data, const regexp::FormalRegExpIteration& iteration) const;
	void Visit(void* data, const regexp::FormalRegExpSymbol& symbol) const;
	void Visit(void* data, const regexp::FormalRegExpEmpty& empty) const;
	void Visit(void* data, const regexp::FormalRegExpEpsilon& epsilon) const;

	static const RegExpEmpty REG_EXP_EMPTY;

public:
	static RegExpEmpty& getInstance() {
		static RegExpEmpty res;
		return res;
	}
};

} /* namespace properties */

} /* namespace regexp */

#endif /* REG_EXP_EMPTY_H_ */
