/*
 * ExactCompare.cpp
 *
 *  Created on: Oct 9, 2014
 *	  Author: Radomir Polach */

#include "ExactCompare.h"

#include "string/LinearString.h"
#include "string/CyclicString.h"

namespace string {

namespace naive {

int ExactCompare::compare(const string::String& u, const string::String& v) {
	int data;
	Accept((void*) &data, u.getData(), v.getData(), ExactCompare::EXACT_COMPARE);
	return data;
}

int ExactCompare::compare(const string::Epsilon&, const string::Epsilon&) {
	return 0;
}

int ExactCompare::compare(const string::LinearString& u, const string::LinearString& v) {
	int n = (int) u.getContent().size(), m = (int)v.getContent().size();
	int k = 0;

	while(k < n && k < m && u.getContent()[k] == v.getContent()[k]) k++;
	if (k == m && k == n) {
		return 0;
	} else if(k == m) {
		return -1;
	} else if(k == n) {
		return 1;
	} else if(u.getContent()[k] < v.getContent()[k]) {
		return -1;
	} else {
		return 1;
	}
}

int ExactCompare::compare(const string::CyclicString& u, const string::CyclicString& v) {
	int n = (int) u.getContent().size(), m = (int)v.getContent().size();
	int i = -1, j = -1, k;

	bool last = 0;
	while(i < n - 1 && j < m - 1)
	{
		k = 1;
		while(k <= n && u.getContent()[(i + k) % n] == v.getContent()[(j + k) % m]) k++;
		if (k > n) return 0;
		last = u.getContent()[(i + k) % n] > v.getContent()[(j + k) % m];
		if (last) i += k; else j += k;
	}
	return last ? 1 : - 1;
}

void ExactCompare::Visit(void* data, const string::Epsilon& u, const string::Epsilon& v) const {
	int & res = *((int*) data);
	res = this->compare(u, v);
}

void ExactCompare::Visit(void* data, const string::LinearString& u, const string::LinearString& v) const {
	int & res = *((int*) data);
	res = this->compare(u, v);
}

void ExactCompare::Visit(void* data, const string::CyclicString& u, const string::CyclicString& v) const {
	int & res = *((int*) data);
	res = this->compare(u, v);
}

const ExactCompare ExactCompare::EXACT_COMPARE;

} /* namespace naive */

} /* namespace string */
