/*
 * BadCharacterShiftTable.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka, Jan Travnicek
 */

#ifndef _BAD_CHARACTER_SHIFT_TABLE_H_
#define _BAD_CHARACTER_SHIFT_TABLE_H_

#include <string/String.h>
#include <string/StringFeatures.h>
#include <common/multipleDispatch.hpp>

#include <set>
#include <map>

namespace stringology {

namespace exact {

/**
 * Computation of BCS table for BMH from MI(E+\eps)-EVY course 2014
 * To get rid of zeros in BCS table we ignore last haystack character
 */
class BadCharacterShiftTable : public std::SingleDispatchFirstStaticParam<std::map<alphabet::Symbol, size_t>, const std::set<alphabet::Symbol>, string::StringBase> {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::map<alphabet::Symbol, size_t> bcs(const string::String& pattern);

	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::map<alphabet::Symbol, size_t> bcs(const std::set<alphabet::Symbol>& alphabet, const string::String& pattern);

	static std::map<alphabet::Symbol, size_t> bcs(const string::LinearString& pattern);

	static std::map<alphabet::Symbol, size_t> bcs(const std::set<alphabet::Symbol>& alphabet, const string::LinearString& pattern);

	static BadCharacterShiftTable& getInstance() {
		static BadCharacterShiftTable res;
		return res;
	}
};

} /* namespace exact */

} /* namespace stringology */

#endif /* _BAD_CHARACTER_SHIFT_TABLE_H_ */
