/*
 * BoyerMooreHorspool.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka
 */

#ifndef _BOYER_MOORE_HORSPOOL_H
#define _BOYER_MOORE_HORSPOOL_H

#include <set>
#include <common/multipleDispatch.hpp>
#include <string/String.h>
#include <string/StringFeatures.h>

namespace stringology {

namespace exact {

/**
 * Implementation of BMH for MI(E+\eps)-EVY course 2014
 * To get rid of zeros in BCS table we ignore last haystack character
 */
class BoyerMooreHorspool : public std::DoubleDispatch<std::set<unsigned>, string::StringBase, string::StringBase> {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::set<unsigned> match(const string::String& subject, const string::String& pattern);

	static std::set<unsigned> match(const string::LinearString& subject, const string::LinearString& pattern);

	static BoyerMooreHorspool& getInstance() {
		static BoyerMooreHorspool res;
		return res;
	}
};

} /* namespace exact */

} /* namespace stringology */

#endif /* _BOYER_MOORE_HORSPOOL_H */
