#include <list>
#include "AutomataUnionTest.h"

#include "automaton/transform/AutomataUnionCartesianProduct.h"
#include "automaton/transform/AutomataUnionEpsilonTransition.h"
#include "automaton/simplify/MinimizeBrzozowski.h"
#include "automaton/simplify/Normalize.h"
#include "automaton/simplify/EpsilonRemoverIncoming.h"
#include "automaton/simplify/Trim.h"
#include "automaton/simplify/Total.h"
#include "automaton/determinize/Determinize.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AutomataUnionTest, "automaton" );
CPPUNIT_TEST_SUITE_REGISTRATION( AutomataUnionTest );

void AutomataUnionTest::setUp() {
}

void AutomataUnionTest::tearDown() {
}

void AutomataUnionTest::testAutomataUnion() {
	// based on Melichar, 2.72

	automaton::State q1a("1"), q2a("2"), q0a("0"), q1b("1'"), q2b("2'"), q0b("0'");
	automaton::State q11("11"), q20("20"), q00("00"), q02("02");
	alphabet::Symbol a(alphabet::symbolFrom('a')), b(alphabet::symbolFrom('b'));

	automaton::DFA m1(q1a);
	automaton::DFA m2(q1b);
	automaton::DFA m3(q11);

	m1.setInputAlphabet({a, b});
	m1.setStates({q1a, q2a, q0a});
	m1.addTransition(q1a, a, q2a);
	m1.addTransition(q1a, b, q0a);
	m1.addTransition(q2a, a, q2a);
	m1.addTransition(q2a, b, q0a);
	m1.addTransition(q0a, a, q0a);
	m1.addTransition(q0a, b, q0a);
	m1.addFinalState(q2a);

	m2.setInputAlphabet({a, b});
	m2.setStates({q1b, q2b});
	m2.addTransition(q1b, b, q2b);
	m2.addTransition(q2b, b, q2b);
	m2.addFinalState(q2b);

	m3.setInputAlphabet({a, b});
	m3.setStates({q11, q20, q00, q02});
	m3.addTransition(q11, a, q20);
	m3.addTransition(q11, b, q02);
	m3.addTransition(q20, a, q20);
	m3.addTransition(q20, b, q00);
	m3.addTransition(q00, a, q00);
	m3.addTransition(q00, b, q00);
	m3.addTransition(q02, a, q00);
	m3.addTransition(q02, b, q02);
	m3.setFinalStates({q20, q02});

	auto u1 = automaton::transform::AutomataUnionEpsilonTransition::unification(automaton::Automaton(m1), automaton::Automaton(m2));
	CPPUNIT_ASSERT_THROW(automaton::transform::AutomataUnionCartesianProduct::unification(automaton::Automaton(m1), automaton::Automaton(m2)), exception::AlibException);
	CPPUNIT_ASSERT_THROW(automaton::transform::AutomataUnionCartesianProduct::unification(automaton::Automaton(automaton::NFA(m1)), automaton::Automaton(m2)), exception::AlibException);
	auto u2 = automaton::transform::AutomataUnionEpsilonTransition::unification(automaton::Automaton(automaton::simplify::Total::total(m1)), automaton::Automaton(automaton::simplify::Total::total(m2)));

	automaton::Automaton umdfa(automaton::simplify::Normalize::normalize(automaton::simplify::Trim::trim(automaton::simplify::MinimizeBrzozowski::minimize(automaton::simplify::EpsilonRemoverIncoming::remove(m3)))));
	automaton::Automaton umdfa1(automaton::simplify::Normalize::normalize(automaton::simplify::Trim::trim(automaton::simplify::MinimizeBrzozowski::minimize(automaton::simplify::EpsilonRemoverIncoming::remove(u1)))));
	automaton::Automaton umdfa2(automaton::simplify::Normalize::normalize(automaton::simplify::Trim::trim(automaton::simplify::MinimizeBrzozowski::minimize(automaton::simplify::EpsilonRemoverIncoming::remove(u2)))));

	CPPUNIT_ASSERT(umdfa1 == umdfa);
	CPPUNIT_ASSERT(umdfa2 == umdfa);

}
