#include <list>
#include "compareTest.h"

#include "string/naive/ExactEqual.h"
#include "string/naive/ExactCompare.h"
#include "string/simplify/NormalizeRotation.h"
#include "string/LinearString.h"
#include "string/CyclicString.h"
#include "string/Epsilon.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( compareTest, "string" );
CPPUNIT_TEST_SUITE_REGISTRATION( compareTest );

void compareTest::setUp() {
}

void compareTest::tearDown() {
}

void compareTest::testCyclicStringCompareBoolean() {
	string::CyclicString str1("alfa");
	string::CyclicString str2("aalf");
	str2.addSymbolsToAlphabet(str1.getAlphabet());
	str1.addSymbolsToAlphabet(str2.getAlphabet());

	CPPUNIT_ASSERT(string::naive::ExactEqual::equals(str1, str2));
}

void compareTest::testCyclicStringCompareInt() {
	string::CyclicString str1("alfa");
	string::CyclicString str2("aalf");
	str2.addSymbolsToAlphabet(str1.getAlphabet());
	str1.addSymbolsToAlphabet(str2.getAlphabet());
	
	CPPUNIT_ASSERT(string::naive::ExactCompare::compare(str1, str2) == 0);


	string::CyclicString str3("ccbcccb");
	string::CyclicString str4("cbcccbc");
	str2.addSymbolsToAlphabet(str1.getAlphabet());
	str1.addSymbolsToAlphabet(str2.getAlphabet());
	
	CPPUNIT_ASSERT(string::naive::ExactCompare::compare(str3, str4) == 0);
}

void compareTest::testCompareCyclic() {
	string::CyclicString cyclic1({alphabet::symbolFrom(0), alphabet::symbolFrom(1), alphabet::symbolFrom(2)});
	string::CyclicString cyclic2({alphabet::symbolFrom(1), alphabet::symbolFrom(2), alphabet::symbolFrom(0)});
	string::CyclicString cyclic3({alphabet::symbolFrom(2), alphabet::symbolFrom(0), alphabet::symbolFrom(1)});

	string::CyclicString cyclic4({alphabet::symbolFrom(2), alphabet::symbolFrom(1), alphabet::symbolFrom(0)});

	string::CyclicString cyclic1n = string::simplify::NormalizeRotation::normalize(cyclic1);
	string::CyclicString cyclic2n = string::simplify::NormalizeRotation::normalize(cyclic2);
	string::CyclicString cyclic3n = string::simplify::NormalizeRotation::normalize(cyclic3);
	string::CyclicString cyclic4n = string::simplify::NormalizeRotation::normalize(cyclic4);


	CPPUNIT_ASSERT(cyclic1n == cyclic2n);
	CPPUNIT_ASSERT(cyclic1n == cyclic3n);
	CPPUNIT_ASSERT(cyclic2n == cyclic3n);

	CPPUNIT_ASSERT(cyclic4n != cyclic1n);
	CPPUNIT_ASSERT(cyclic4n != cyclic2n);
	CPPUNIT_ASSERT(cyclic4n != cyclic3n);
}

