/*
 * Alphabet.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef ALPHABET_H_
#define ALPHABET_H_

#include <set>

#include "Symbol.h"

namespace alphabet {

/**
 * Represents alphabet of symbols.
 */
class Alphabet {
	std::set<alphabet::Symbol> m_Data;

public:
	/**
	 * @return List of symbols in the alphabet.
	 */
	const std::set<alphabet::Symbol>& getContent() const;
	
	/**
	 * @param symbol to insert
	 */
	void insertSymbol(alphabet::Symbol&& symbol);
	
	/**
	 * @param symbol to insert
	 */
	void insertSymbol(const alphabet::Symbol& symbol);

	/**
	 * @return true if alphabet is an empty
	 */
	bool isEmpty() const;
	
	/**
	 * @param other other alphabet
	 */
	bool operator==(const Alphabet& other) const;
	
	/**
	 * @param other other alphabet
	 */
	bool operator<(const Alphabet& other) const;

	/**
	 * Prints XML representation of the Alphabet to the output stream.
	 * @param out output stream to which print the Alphabet
	 * @param string Alphabet to print
	 */
	friend std::ostream& operator<<(std::ostream& out, const Alphabet& string);

};

} /* namespace alphabet */

#endif /* ALPHABET_H_ */
