/*
 * EndSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef END_SYMBOL_H_
#define END_SYMBOL_H_

#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class EndSymbol : public std::acceptor<EndSymbol, VisitableSymbolBase, std::acceptor<EndSymbol, alib::VisitableObjectBase, SymbolBase> > {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit EndSymbol();

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const EndSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	static EndSymbol END;

	const static std::string XML_TAG_NAME;

	static EndSymbol parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual SymbolBase* next() const;
	virtual SymbolBase* inc() &&;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::EndSymbol> {
	int operator()(const alphabet::EndSymbol& first, const alphabet::EndSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* END_SYMBOL_H_ */
