/*
 * LabeledSymbol.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Martin Zak
 */

#include "LabeledSymbol.h"
#include "../sax/FromXMLParserHelper.h"
#include "Symbol.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace alphabet {

LabeledSymbol::LabeledSymbol(int number) : label(label::labelFrom(number)) {

}

LabeledSymbol::LabeledSymbol(char character) : label(label::labelFrom(character)) {

}

LabeledSymbol::LabeledSymbol(std::string label) : label(label::labelFrom(std::move(label))) {

}

LabeledSymbol::LabeledSymbol(label::Label label) : label(std::move(label)) {

}

SymbolBase* LabeledSymbol::clone() const {
	return new LabeledSymbol(*this);
}

SymbolBase* LabeledSymbol::plunder() && {
	return new LabeledSymbol(std::move(*this));
}

const label::Label& LabeledSymbol::getLabel() const {
	return label;
}

label::Label& LabeledSymbol::getLabel() {
	return label;
}

int LabeledSymbol::compare(const LabeledSymbol& other) const {
	return this->label.getData().compare(other.label.getData());
}

void LabeledSymbol::operator>>(std::ostream& out) const {
	out << "(LabeledSymbol " << this->label << ")";
}

LabeledSymbol::operator std::string () const {
	return (std::string) label;
}

const std::string LabeledSymbol::XML_TAG_NAME = "LabeledSymbol";

LabeledSymbol LabeledSymbol::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, LabeledSymbol::XML_TAG_NAME);
	label::Label data = alib::xmlApi<label::Label>::parse(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, LabeledSymbol::XML_TAG_NAME);
	return LabeledSymbol(std::move(data));
}

void LabeledSymbol::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(LabeledSymbol::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<label::Label>::compose(out, label);
	out.emplace_back(LabeledSymbol::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

SymbolBase* LabeledSymbol::next() const {
	return new LabeledSymbol(label.next());
}

SymbolBase* LabeledSymbol::inc() && {
	label.inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

xmlApi<alphabet::Symbol>::ParserRegister<alphabet::LabeledSymbol> labeledSymbolParserRegister = xmlApi<alphabet::Symbol>::ParserRegister<alphabet::LabeledSymbol>(alphabet::LabeledSymbol::XML_TAG_NAME, alphabet::LabeledSymbol::parse);
xmlApi<alib::Object>::ParserRegister<alphabet::LabeledSymbol> labeledSymbolParserRegister2 = xmlApi<alib::Object>::ParserRegister<alphabet::LabeledSymbol>(alphabet::LabeledSymbol::XML_TAG_NAME, alphabet::LabeledSymbol::parse);

} /* namespace alib */
