/*
 * StartSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef START_SYMBOL_H_
#define START_SYMBOL_H_

#include "SymbolBase.h"

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class StartSymbol : public std::acceptor<StartSymbol, VisitableSymbolBase, std::acceptor<StartSymbol, alib::VisitableObjectBase, SymbolBase> > {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit StartSymbol();

	virtual SymbolBase* clone() const;
	virtual SymbolBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const StartSymbol& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	static StartSymbol START;

	const static std::string XML_TAG_NAME;

	static StartSymbol parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual SymbolBase* next() const;
	virtual SymbolBase* inc() &&;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::StartSymbol> {
	int operator()(const alphabet::StartSymbol& first, const alphabet::StartSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* START_SYMBOL_H_ */
