/*
 * SymbolPairSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_PAIR_SYMBOL_H_
#define SYMBOL_PAIR_SYMBOL_H_

#include "Symbol.h"
#include <pair>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class SymbolPairSymbol : public std::acceptor<SymbolPairSymbol, VisitableSymbolBase, std::acceptor<SymbolPairSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	std::pair<Symbol, Symbol> symbol;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit SymbolPairSymbol(std::pair<Symbol, Symbol> symbol);

	virtual SymbolBase* clone() const;

	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::pair<Symbol, Symbol>& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const SymbolPairSymbol& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static SymbolPairSymbol parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual SymbolBase* next() const;
	virtual SymbolBase* inc() &&;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::SymbolPairSymbol> {
	int operator()(const alphabet::SymbolPairSymbol& first, const alphabet::SymbolPairSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SYMBOL_PAIR_SYMBOL_H_ */

