/*
 * UniqueSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_ID_SYMBOL_H_
#define SYMBOL_ID_SYMBOL_H_

#include "Symbol.h"
#include "../primitive/Integer.h"

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class UniqueSymbol : public std::acceptor<UniqueSymbol, VisitableSymbolBase, std::acceptor<UniqueSymbol, alib::VisitableObjectBase, SymbolBase> > {
protected:
	Symbol symbol;
	primitive::Integer id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueSymbol(Symbol symbol, primitive::Integer id);

	virtual SymbolBase* clone() const;

	virtual SymbolBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const Symbol& getSymbol() const;

	const primitive::Integer& getId() const;

	primitive::Integer& getId();

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const UniqueSymbol& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static UniqueSymbol parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual SymbolBase* next() const;
	virtual SymbolBase* inc() &&;
};

} /* namespace alphabet */

namespace std {

template<>
struct compare<alphabet::UniqueSymbol> {
	int operator()(const alphabet::UniqueSymbol& first, const alphabet::UniqueSymbol& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* SYMBOL_ID_SYMBOL_H_ */

