/*
 * BlankSymbolInputTapeAlphabet.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef BLANK_SYMBOL_INPUT_TAPE_ALPHABET_H_
#define BLANK_SYMBOL_INPUT_TAPE_ALPHABET_H_

#include <set>
#include "../../alphabet/Symbol.h"

namespace automaton {

class BlankSymbolInputTapeAlphabet {
protected:
	std::set<alphabet::Symbol> tapeAlphabet;
	std::set<alphabet::Symbol> inputAlphabet;
	alphabet::Symbol blankSymbol;
public:
	BlankSymbolInputTapeAlphabet(alphabet::Symbol blank);

	/**
	 * Adds input symbol to input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol already exists
	 */
	bool addInputSymbol(alphabet::Symbol symbol);

	/**
	 * Sets input symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throws AutomatonException when symbol already exists
	 */
	void setInputAlphabet(std::set<alphabet::Symbol> symbol);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is not present in input alphabet
	 * or when symbol is part of the transition
	 */
	virtual bool removeInputSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getInputAlphabet() const;

	/**
	 * Adds symbol to the tape alphabet.
	 * @param symbol Symbol to add
	 * @throw AutomatonException when Symbol is already present in tape alphabet
	 */
	bool addTapeSymbol(alphabet::Symbol symbol);

	/**
	 * Sets tape symbols of the automaton.
	 * @param symbols Symbols to set
	 * @throw AutomatonException when Symbol is already present in tape alphabet
	 */
	void setTapeAlphabet(std::set<alphabet::Symbol> symbols);

	/**
	 * Removes symbol from the tape alphabet.
	 * @param symbol Symbol to remove
	 * @throw AutomatonException when Symbol is not present in the tape alphabet,
	 * when it is used in transition or when it is present in input alphabet
	 */
	virtual bool removeTapeSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return tape alphabet
	 */
	const std::set<alphabet::Symbol>& getTapeAlphabet() const;

	/**
	 * Sets the blank symbol = symbol representing empty cell of the tape
	 * @param symbol Symbol to set
	 */
	void setBlankSymbol(alphabet::Symbol symbol);

	/**
	 * @return symbol representing empty cell of the tape
	 */
	const alphabet::Symbol& getBlankSymbol() const;

};

} /* namespace automaton */

#endif /* BLANK_SYMBOL_INPUT_TAPE_ALPHABET_H_ */

