/*
 * InputAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "InputAlphabet.h"

#include <algorithm>
#include "../AutomatonException.h"

namespace automaton {

const std::set<alphabet::Symbol>& InputAlphabet::getInputAlphabet() const {
	return inputAlphabet;
}

bool InputAlphabet::addInputSymbol(alphabet::Symbol symbol) {
	return inputAlphabet.insert(std::move(symbol)).second;
}

void InputAlphabet::addInputSymbols(std::set<alphabet::Symbol> symbols) {
	return inputAlphabet.insert(std::make_set_move_iterator(symbols.begin()), std::make_set_move_iterator(symbols.end()));
}

void InputAlphabet::setInputAlphabet(std::set<alphabet::Symbol> newSymbols) {
	std::set<alphabet::Symbol> removed;
	std::set_difference(inputAlphabet.begin(), inputAlphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::Symbol& removedSymbol : removed) {
		removeInputSymbol(removedSymbol);
	}

	inputAlphabet = std::move(newSymbols);
}

} /* namespace automaton */

