/*
 * MultiInitialSymbols.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef MULTI_INITIAL_SYMBOLS_PUSHDOWN_STORE_ALPHABET_H_
#define MULTI_INITIAL_SYMBOLS_PUSHDOWN_STORE_ALPHABET_H_

#include <set>
#include "../../alphabet/Symbol.h"
#include "PushdownStoreAlphabet.h"

namespace automaton {

/**
 * Push Down Automaton
 */
class MultiInitialSymbolsPushdownStoreAlphabet : public PushdownStoreAlphabet {
protected:
	std::set<alphabet::Symbol> initialSymbols;
public:
	/**
	 * Adds initial symbol. Initial symbols are symbols that are pushed
	 * to the stack when MultiInitialSymbols is created.
	 * @param start new initial symbol
	 * @throws AutomatonException when symbol is not present in the stack alphabet or it is already in the set
	 */
	bool addInitialSymbol(alphabet::Symbol start);

	/**
	 * Adds initial symbol. Initial symbols are symbols that are pushed
	 * to the stack when MultiInitialSymbols is created.
	 * @param start new initial symbol
	 * @throws AutomatonException when symbol is not present in the set of initial symbols
	 */
	bool removeInitialSymbol(const alphabet::Symbol& start);

	/**
	 * Sets initial symbols. Initial symbols are symbols that are pushed
	 * to the stack when MultiInitialSymbols is created.
	 * @param symbols new initial symbols
	 * @throws AutomatonException when any of symbols is not present in the stack alphabet
	 */
	void setInitialSymbols(std::set<alphabet::Symbol> symbols);

	/**
	 * @return list of start symbols
	 */
	const std::set<alphabet::Symbol>& getInitialSymbols() const;

};

} /* namespace automaton */

#endif /* MULTI_INITIAL_SYMBOLS_PUSHDOWN_STORE_ALPHABET_H_ */
