/*
 * Base.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Jan Travnicek
 */

#ifndef BASE_H_
#define BASE_H_

#include <typeinfo>
#include <ostream>
#include <stdexcept>

namespace alib {

template<typename T>
class base {
public:
	virtual ~base() noexcept {}

	virtual long long selfTypeId() const = 0;

	virtual T* clone() const = 0;

	virtual T* plunder() && = 0;

	bool operator!=(const T& other) const {
		return this->compare(other) != 0;
	}

	bool operator==(const T & other) const {
		return this->compare(other) == 0;
	}

	bool operator<(const T & other) const {
		return this->compare(other) < 0;
	}

	bool operator<=(const T & other) const {
		return this->compare(other) <= 0;
	}

	bool operator>(const T & other) const {
		return this->compare(other) > 0;
	}

	bool operator>=(const T & other) const {
		return this->compare(other) >= 0;
	}

	virtual int compare(const T & other) const = 0;

	template<typename R, typename S> //TODO what if someone want to control the ordering?
	static int compare(const R& first, const S& second) {
		if(first.selfTypeId() < second.selfTypeId())
			return -1;
		else if(first.selfTypeId() > second.selfTypeId())
			return 1;
		else
			return first.compare((const R&) second);
	}

	friend std::ostream& operator<<(std::ostream& os, const T & instance) {
		instance >> os;
		return os;
	}

	virtual void operator>>(std::ostream&) const = 0;

	virtual explicit operator std::string () const = 0;

	template<typename R>
	static long long typeId(const R&) {
		return (long long) &base::typeId<R>;
	}
};

} /* namespace alib */

#endif /* BASE_H_ */
