/*
 * EpsilonFreeCFG.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_FREE_CFG_H_
#define EPSILON_FREE_CFG_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Epsilon free context free grammar. Type 2 in Chomsky hierarchy. Produces context free languages.
 */
class EpsilonFreeCFG : public std::acceptor<EpsilonFreeCFG, VisitableGrammarBase, std::acceptor<EpsilonFreeCFG, alib::VisitableObjectBase, GrammarBase> >, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> rules;
	bool generatesEpsilon;

public:
	explicit EpsilonFreeCFG(alphabet::Symbol initialSymbol);

	explicit EpsilonFreeCFG(std::set<alphabet::Symbol> nonTerminalSymbols, std::set<alphabet::Symbol> terminalSymbols, alphabet::Symbol initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(alphabet::Symbol leftHandSide, std::vector<alphabet::Symbol> rightHandSide);

	const std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> & getRules() const;

	bool removeRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool addRawRule(alphabet::Symbol leftHandSide, std::vector<alphabet::Symbol> rightHandSide);

	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> getRawRules() const;

	bool removeRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	void setGeneratesEpsilon(bool genEps);
	bool getGeneratesEpsilon() const;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const EpsilonFreeCFG& other) const;

	virtual void operator>>(std::ostream& os) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static EpsilonFreeCFG parse(std::deque<sax::Token>::iterator& input);
	static void parseRule(std::deque<sax::Token>::iterator& input, EpsilonFreeCFG& grammar);

	void compose(std::deque<sax::Token>& out) const;
	void composeRules(std::deque<sax::Token>& out) const;
};

} /* namespace grammar */

namespace std {

template<>
struct compare<grammar::EpsilonFreeCFG> {
	int operator()(const grammar::EpsilonFreeCFG& first, const grammar::EpsilonFreeCFG& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* EPSILON_FREE_CFG_H_ */
