/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef I_DIRECTED_GRAPH_H_
#define I_DIRECTED_GRAPH_H_

#include <set>

#include "../common/Node.h"
#include "DirectedEdge.h"

namespace graph {

class IDirectedGraph
{
public:
	virtual ~IDirectedGraph() noexcept;

	virtual std::set<Node> allNodes() const = 0;
	virtual std::set<DirectedEdge> allEdges() const = 0;

	virtual bool hasNode(const Node &node) const = 0;
	virtual bool hasEdge(const DirectedEdge &edge) const = 0;

	virtual bool hasEdge(const Node &from, const Node &to) const = 0;
	virtual std::set<DirectedEdge> findEdges(const Node &from, const Node &to) const = 0;

	virtual std::set<Node> neighbors(const Node &node) const = 0;
	virtual std::set<DirectedEdge> neighborEdges(const Node &node) const = 0;

	virtual bool addNode(const Node &node) = 0;
	virtual bool removeNode(const Node &node) = 0;

	virtual bool addEdge(const DirectedEdge &edge) = 0;
	virtual bool removeEdge(const DirectedEdge &edge) = 0;

	virtual void copy(IDirectedGraph *other) = 0;
	virtual int compare(IDirectedGraph *other) const = 0;

	explicit operator std::string() const;
};

} // namespace graph

#endif // I_DIRECTED_GRAPH_H_
