/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef ADJACENCY_LIST_UNDIRECTED_GRAPH_H_
#define ADJACENCY_LIST_UNDIRECTED_GRAPH_H_

#include <list>
#include <unordered_map>

#include "IUndirectedGraph.h"

namespace graph {

/*
 *  AdjacencyList representation
 *
 *  unordered_map:
 *    [ n1 ] -> list [ n2, n2, n3, n4 ]
 *    [ n2 ] -> list [ n1, n1, n4 ]      < multi-edge
 *    [ n3 ] -> list [ n1, n4 ]
 *    [ n4 ] -> list [ n1, n2, n3 ]
 *
 *  edges:
 *    n1 <-> n2, n1 <-> n2, n1 <-> n3, n1 <-> n4
 *    n2 <-> n4
 *    n3 <-> n4
 *
 *  undirected edge n1 <-> n2 is represented as
 *  directed edge n1 -> n2 and directed edge n2 -> n1
 */

class AdjacencyListUndirectedGraph : public IUndirectedGraph
{
public:
	std::set<Node> allNodes() const override;
	std::set<UndirectedEdge> allEdges() const override;

	bool hasNode(const Node &node) const override;
	bool hasEdge(const UndirectedEdge &edge) const override;

	bool hasEdge(const Node &first, const Node &second) const override;
	std::set<UndirectedEdge> findEdges(const Node &first, const Node &second) const override;

	std::set<Node> neighbors(const Node &node) const override;
	std::set<UndirectedEdge> neighborEdges(const Node &node) const override;

	bool addNode(const Node &node) override;
	bool removeNode(const Node &node) override;

	bool addEdge(const UndirectedEdge &edge) override;
	bool removeEdge(const UndirectedEdge &edge) override;

	void copy(IUndirectedGraph *other) override;
	int compare(IUndirectedGraph *other) const override;

private:
	std::set<UndirectedEdge> edges;
	std::unordered_map<Node, std::list<Node>> adj;
};

} // namespace graph

#endif // ADJACENCY_LIST_DIRECTED_GRAPH_H_
