/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef I_UNDIRECTED_GRAPH_H_
#define I_UNDIRECTED_GRAPH_H_

#include <set>

#include "../common/Node.h"
#include "UndirectedEdge.h"

namespace graph {

class IUndirectedGraph
{
public:
	virtual ~IUndirectedGraph() noexcept;

	virtual std::set<Node> allNodes() const = 0;
	virtual std::set<UndirectedEdge> allEdges() const = 0;

	virtual bool hasNode(const Node &node) const = 0;
	virtual bool hasEdge(const UndirectedEdge &edge) const = 0;

	virtual bool hasEdge(const Node &first, const Node &second) const = 0;
	virtual std::set<UndirectedEdge> findEdges(const Node &first, const Node &second) const = 0;

	virtual std::set<Node> neighbors(const Node &node) const = 0;
	virtual std::set<UndirectedEdge> neighborEdges(const Node &node) const = 0;

	virtual bool addNode(const Node &node) = 0;
	virtual bool removeNode(const Node &node) = 0;

	virtual bool addEdge(const UndirectedEdge &edge) = 0;
	virtual bool removeEdge(const UndirectedEdge &edge) = 0;

	virtual void copy(IUndirectedGraph *other) = 0;
	virtual int compare(IUndirectedGraph *other) const = 0;

	explicit operator std::string() const;
};

} // namespace graph

#endif // I_UNDIRECTED_GRAPH_H_
