/*
 * HexavigesimalLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "HexavigesimalLabel.h"
#include "../sax/FromXMLParserHelper.h"
#include "Label.h"
#include "../object/Object.h"
#include "../XmlApi.hpp"

namespace label {

HexavigesimalLabel::HexavigesimalLabel(int hexavigesimal) : hexavigesimal(hexavigesimal) {

}

LabelBase* HexavigesimalLabel::clone() const {
	return new HexavigesimalLabel(*this);
}

LabelBase* HexavigesimalLabel::plunder() && {
	return new HexavigesimalLabel(std::move(*this));
}

int HexavigesimalLabel::getData() const {
	return hexavigesimal;
}

void HexavigesimalLabel::setData(int data) {
	this->hexavigesimal = data;
}

int HexavigesimalLabel::compare(const HexavigesimalLabel& other) const {
	return hexavigesimal - other.hexavigesimal;
}

void HexavigesimalLabel::operator>>(std::ostream& out) const {
	out << "(HexavigesimalLabel " << std::toBase26(hexavigesimal) << ")";
}

HexavigesimalLabel::operator std::string() const {
	return std::toBase26(hexavigesimal);
}

const std::string HexavigesimalLabel::XML_TAG_NAME = "HexavigesimalLabel";

HexavigesimalLabel HexavigesimalLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, HexavigesimalLabel::XML_TAG_NAME);
	HexavigesimalLabel data(std::stoi(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER)));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, HexavigesimalLabel::XML_TAG_NAME);
	return data;
}

void HexavigesimalLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(HexavigesimalLabel::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::itos(hexavigesimal), sax::Token::TokenType::CHARACTER);
	out.emplace_back(HexavigesimalLabel::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

LabelBase* HexavigesimalLabel::next() const {
	return new HexavigesimalLabel(hexavigesimal + 1);
}

LabelBase* HexavigesimalLabel::inc() && {
	hexavigesimal++;
	return NULL;
}

} /* namespace label */

namespace alib {

xmlApi<label::Label>::ParserRegister<label::HexavigesimalLabel> hexavigesimalLabelParserRegister = xmlApi<label::Label>::ParserRegister<label::HexavigesimalLabel>(label::HexavigesimalLabel::XML_TAG_NAME, label::HexavigesimalLabel::parse);
xmlApi<alib::Object>::ParserRegister<label::HexavigesimalLabel> hexavigesimalLabelParserRegister2 = xmlApi<alib::Object>::ParserRegister<label::HexavigesimalLabel>(label::HexavigesimalLabel::XML_TAG_NAME, label::HexavigesimalLabel::parse);

} /* namespace alib */
