/*
 * HexavigesimalLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef HEXAVIGESIMAL_LABEL_H_
#define HEXAVIGESIMAL_LABEL_H_

#include <hexavigesimal>

#include "LabelBase.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class HexavigesimalLabel : public std::acceptor<HexavigesimalLabel, VisitableLabelBase, std::acceptor<HexavigesimalLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	int hexavigesimal;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit HexavigesimalLabel(int hexavigesimal);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	int getData() const;

	void setData(int data);

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const HexavigesimalLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static HexavigesimalLabel parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual LabelBase* next() const;
	virtual LabelBase* inc() &&;
};

} /* namespace label */

namespace std {

template<>
struct compare<label::HexavigesimalLabel> {
	int operator()(const label::HexavigesimalLabel& first, const label::HexavigesimalLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* HEXAVIGESIMAL_LABEL_H_ */

