/*
 * LabelPairLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_PAIR_LABEL_H_
#define LABEL_PAIR_LABEL_H_

#include "Label.h"
#include <pair>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelPairLabel : public std::acceptor<LabelPairLabel, VisitableLabelBase, std::acceptor<LabelPairLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	std::pair<Label, Label> label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabelPairLabel(std::pair<Label, Label> label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::pair<Label, Label>& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const LabelPairLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static LabelPairLabel parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual LabelBase* next() const;
	virtual LabelBase* inc() &&;
};

} /* namespace label */

namespace std {

template<>
struct compare<label::LabelPairLabel> {
	int operator()(const label::LabelPairLabel& first, const label::LabelPairLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LABEL_PAIR_LABEL_H_ */

