/*
 * LabelSetLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_SET_LABEL_H_
#define LABEL_SET_LABEL_H_

#include "Label.h"
#include <set>

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class LabelSetLabel : public std::acceptor<LabelSetLabel, VisitableLabelBase, std::acceptor<LabelSetLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	std::set<Label> label;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit LabelSetLabel(std::set<Label> label);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::set<Label>& getData() const;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const LabelSetLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static LabelSetLabel parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual LabelBase* next() const;
	virtual LabelBase* inc() &&;
};

} /* namespace label */

namespace std {

template<>
struct compare<label::LabelSetLabel> {
	int operator()(const label::LabelSetLabel& first, const label::LabelSetLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LABEL_SET_LABEL_H_ */

