/*
 * PrimitiveLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_LABEL_H_
#define PRIMITIVE_LABEL_H_

#include "../primitive/Primitive.h"
#include "LabelBase.h"
#include "Label.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class PrimitiveLabel : public std::acceptor<PrimitiveLabel, VisitableLabelBase, std::acceptor<PrimitiveLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	primitive::Primitive primitive;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit PrimitiveLabel(primitive::Primitive primitive);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const primitive::Primitive& getData() const;

	primitive::Primitive& getData();

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const PrimitiveLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static PrimitiveLabel parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual LabelBase* next() const;
	virtual LabelBase* inc() &&;
};

} /* namespace label */

namespace std {

template<>
struct compare<label::PrimitiveLabel> {
	int operator()(const label::PrimitiveLabel& first, const label::PrimitiveLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* PRIMITIVE_LABEL_H_ */

