/*
 * UniqueLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_ID_LABEL_H_
#define LABEL_ID_LABEL_H_

#include "Label.h"
#include "../primitive/Integer.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class UniqueLabel : public std::acceptor<UniqueLabel, VisitableLabelBase, std::acceptor<UniqueLabel, alib::VisitableObjectBase, LabelBase> > {
protected:
	Label label;
	primitive::Integer id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueLabel(Label label, primitive::Integer id);

	virtual LabelBase* clone() const;

	virtual LabelBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const Label& getLabel() const;

	const primitive::Integer& getId() const;

	primitive::Integer& getId();

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const UniqueLabel& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static UniqueLabel parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;

	virtual LabelBase* next() const;
	virtual LabelBase* inc() &&;
};

} /* namespace label */

namespace std {

template<>
struct compare<label::UniqueLabel> {
	int operator()(const label::UniqueLabel& first, const label::UniqueLabel& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* LABEL_ID_LABEL_H_ */

