/*
 * ObjectBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef OBJECT_BASE_H_
#define OBJECT_BASE_H_

#include "../common/visitor.hpp"
#include <compare>
#include <memory>
#include <deque>
#include "../common/base.hpp"
#include "../sax/Token.h"


#include "ObjectFeatures.h"
#include "../exception/ExceptionFeatures.h"
#include "../automaton/AutomatonFeatures.h"
#include "../grammar/GrammarFeatures.h"
#include "../graph/GraphFeatures.h"
#include "../label/LabelFeatures.h"
#include "../regexp/RegExpFeatures.h"
#include "../string/StringFeatures.h"
#include "../alphabet/SymbolFeatures.h"
#include "../container/ContainerFeatures.h"
#include "../primitive/PrimitiveFeatures.h"
#include "../tree/TreeFeatures.h"

namespace alib {

// ----------------------------------------------------------------------------------------------------------------------

typedef std::tuple< tree::RankedTree, tree::RankedPattern, tree::PrefixRankedTree, tree::PrefixRankedBarTree, tree::PrefixRankedPattern, tree::PrefixRankedBarPattern, tree::UnrankedTree, tree::UnrankedPattern
	> TreeTypes;

typedef std::tuple< primitive::String, primitive::Integer, primitive::Character, primitive::Unsigned, primitive::Bool
	> PrimitiveTypes;

typedef std::tuple< container::ObjectsSet, container::ObjectsVector, container::ObjectsPair, container::ObjectsMap
	> ContainerTypes;

typedef std::tuple< alphabet::EndSymbol, alphabet::LabeledSymbol, alphabet::BlankSymbol, alphabet::BottomOfTheStackSymbol, alphabet::RankedSymbol, alphabet::BarSymbol, alphabet::VariablesBarSymbol, alphabet::SubtreeWildcardSymbol, alphabet::SymbolPairSymbol, alphabet::SymbolSetSymbol, alphabet::UniqueSymbol, alphabet::StartSymbol
	> SymbolTypes;

typedef std::tuple< string::Epsilon, string::LinearString, string::CyclicString
	> StringTypes;

typedef std::tuple< regexp::UnboundedRegExp, regexp::FormalRegExp
	> RegExpTypes;

typedef std::tuple< label::PrimitiveLabel, label::HexavigesimalLabel, label::ObjectLabel, label::LabelSetLabel, label::LabelPairLabel, label::UniqueLabel
	> LabelTypes;

typedef std::tuple< graph::DirectedGraph, graph::UndirectedGraph
	> GraphTypes;

typedef std::tuple< grammar::LeftLG, grammar::LeftRG, grammar::RightLG, grammar::RightRG, grammar::LG, grammar::CFG, grammar::EpsilonFreeCFG, grammar::CNF, grammar::GNF, grammar::CSG, grammar::NonContractingGrammar, grammar::ContextPreservingUnrestrictedGrammar, grammar::UnrestrictedGrammar
	> GrammarTypes;

typedef std::tuple< automaton::DFA, automaton::NFA, automaton::MultiInitialStateNFA, automaton::EpsilonNFA, automaton::CompactNFA, automaton::ExtendedNFA, automaton::DFTA, automaton::NFTA, automaton::DPDA, automaton::SinglePopDPDA, automaton::InputDrivenDPDA, automaton::VisiblyPushdownDPDA, automaton::RealTimeHeightDeterministicDPDA, automaton::NPDA, automaton::SinglePopNPDA, automaton::InputDrivenNPDA, automaton::VisiblyPushdownNPDA, automaton::RealTimeHeightDeterministicNPDA, automaton::OneTapeDTM
	> AutomatonTypes;

typedef std::tuple< Void, exception::AlibException
	> OtherTypes;

// --------------------------------------------------------------------------------------------------------------------

template<typename ... Ts>
class types_merger {
};

template<typename ... TreeTypes, typename ... PrimitiveTypes, typename ... ContainerTypes, typename ... SymbolTypes, typename ... StringTypes, typename ... RegExpTypes, typename ... LabelTypes, typename ... GraphTypes, typename ... GrammarTypes, typename ... AutomatonTypes, typename ... OtherTypes>
class types_merger<std::tuple<TreeTypes...>, std::tuple<PrimitiveTypes...>, std::tuple<ContainerTypes...>, std::tuple<SymbolTypes...>, std::tuple<StringTypes...>, std::tuple<RegExpTypes...>, std::tuple<LabelTypes...>, std::tuple<GraphTypes...>, std::tuple<GrammarTypes...>, std::tuple<AutomatonTypes...>, std::tuple<OtherTypes...>> {
public:
	typedef std::tuple<
			OtherTypes..., AutomatonTypes..., GrammarTypes..., GraphTypes..., LabelTypes..., RegExpTypes..., StringTypes..., SymbolTypes..., ContainerTypes..., PrimitiveTypes..., TreeTypes...
		> MergedTypes;

};

typedef types_merger<TreeTypes, PrimitiveTypes, ContainerTypes, SymbolTypes, StringTypes, RegExpTypes, LabelTypes, GraphTypes, GrammarTypes, AutomatonTypes, OtherTypes>::MergedTypes Types;

// ----------------------------------------------------------------------------------------------------------------------

class ObjectBase;

template<typename T>
class acceptor_base_helper {
};

template<typename ... Ts>
class acceptor_base_helper< std::tuple< Ts ... > > :
	public std::acceptor_base<
			ObjectBase, Ts ...
	> {
};

typedef acceptor_base_helper< Types > VisitableObjectBase;

// ----------------------------------------------------------------------------------------------------------------------

class ObjectBase : public alib::base<ObjectBase>, public VisitableObjectBase, public std::cow_shared_ptr_base {
public:
	virtual void compose(std::deque<sax::Token>& out) const = 0;
};

// ----------------------------------------------------------------------------------------------------------------------

} /* namespace alib */

namespace std {

template<>
struct compare<alib::ObjectBase> {
	int operator()(const alib::ObjectBase& first, const alib::ObjectBase& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* OBJECT_BASE_H_ */
