/*
 * Void.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef VOID_H_
#define VOID_H_

#include "ObjectBase.h"
#include <deque>
#include "../sax/Token.h"

namespace alib {

/**
 * Represents void.
 */
class Void : public std::acceptor<Void, VisitableObjectBase, ObjectBase> {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit Void();

	virtual ObjectBase* clone() const;
	virtual ObjectBase* plunder() &&;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const Void& other) const;

	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	static Void VOID;

	const static std::string XML_TAG_NAME;

	static Void parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;
};

} /* namespace alib */

namespace std {

template<>
struct compare<alib::Void> {
	int operator()(const alib::Void& first, const alib::Void& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* VOID_H_ */
