/*
 * Bool.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_BOOL_H_
#define PRIMITIVE_BOOL_H_

#include "PrimitiveBase.h"
#include "../XmlApi.hpp"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Bool : public std::acceptor<Bool, VisitablePrimitiveBase, std::acceptor<Bool, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	bool data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Bool(bool data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	bool getData() const;

	/**
	 * sets new content of primitive
	 */
	void setData(bool data);

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const Bool& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string() const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static Bool parse(std::deque<sax::Token>::iterator& input);
	static bool parseRaw(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;
	static void compose(std::deque<sax::Token>& out, bool primitive);

	Bool next() const;
	virtual void inc();
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::Bool> {
	int operator()(const primitive::Bool& first, const primitive::Bool& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

namespace alib {

template<>
struct xmlApi<bool> {
	static bool parse(std::deque<sax::Token>::iterator& input);
	static bool first(const std::deque<sax::Token>::const_iterator& input);
	static void compose(std::deque<sax::Token>& output, bool data);
};

} /* namespace alib */

#endif /* PRIMITIVE_BOOL_H_ */
