/*
 * Character.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_CHARACTER_H_
#define PRIMITIVE_CHARACTER_H_

#include "PrimitiveBase.h"
#include "../XmlApi.hpp"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Character : public std::acceptor<Character, VisitablePrimitiveBase, std::acceptor<Character, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	char data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Character(char data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	char getData() const;

	/**
	 * sets new content of primitive
	 */
	void setData(char data);

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const Character& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static Character parse(std::deque<sax::Token>::iterator& input);
	static char parseRaw(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;
	static void compose(std::deque<sax::Token>& out, char primitive);

	Character next() const;
	virtual void inc();
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::Character> {
	int operator()(const primitive::Character& first, const primitive::Character& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

namespace alib {

template<>
struct xmlApi<char> {
	static char parse(std::deque<sax::Token>::iterator& input);
	static bool first(const std::deque<sax::Token>::const_iterator& input);
	static void compose(std::deque<sax::Token>& output, char data);
};

} /* namespace alib */

#endif /* PRIMITIVE_CHARACTER_H_ */

