/*
 * Integer.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Integer.h"
#include "../sax/FromXMLParserHelper.h"
#include "Primitive.h"
#include "../object/Object.h"

namespace primitive {

Integer::Integer(int data) : data(data) {

}

PrimitiveBase* Integer::clone() const {
	return new Integer(*this);
}

PrimitiveBase* Integer::plunder() && {
	return new Integer(std::move(*this));
}

int Integer::getData() const {
	return data;
}

void Integer::setData(int data) {
	this->data = data;
}

int Integer::compare(const Integer& other) const {
	return data - other.data;
}

void Integer::operator>>(std::ostream& out) const {
	out << "(Integer " << data << ")";
}

Integer::operator std::string() const {
	return std::to_string(data);
}

const std::string Integer::XML_TAG_NAME = "Integer";

Integer Integer::parse(std::deque<sax::Token>::iterator& input) {
	return Integer(parseRaw(input));
}

int Integer::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Integer::XML_TAG_NAME);
	int data = std::stoi(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Integer::XML_TAG_NAME);
	return data;
}

void Integer::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Integer::compose(std::deque<sax::Token>& out, int primitive) {
	out.emplace_back(Integer::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::itos(primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Integer::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

Integer Integer::next() const {
	return Integer(data + 1);
}

void Integer::inc() {
	data++;
}

} /* namespace primitive */

namespace alib {

int xmlApi<int>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Integer::parseRaw(input);
}

bool xmlApi<int>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, primitive::Integer::XML_TAG_NAME);
}

void xmlApi<int>::compose(std::deque<sax::Token>& output, int data) {
	primitive::Integer::compose(output, data);
}

xmlApi<primitive::Primitive>::ParserRegister<primitive::Integer> integerParserRegister = xmlApi<primitive::Primitive>::ParserRegister<primitive::Integer>(primitive::Integer::XML_TAG_NAME, primitive::Integer::parse);
xmlApi<alib::Object>::ParserRegister<primitive::Integer> integerParserRegister2 = xmlApi<alib::Object>::ParserRegister<primitive::Integer>(primitive::Integer::XML_TAG_NAME, primitive::Integer::parse);

} /* namespace alib */

