/*
 * StringLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_STRING_H_
#define PRIMITIVE_STRING_H_

#include <string>
#include "PrimitiveBase.h"
#include "../XmlApi.hpp"

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class String : public std::acceptor<String, VisitablePrimitiveBase, std::acceptor<String, alib::VisitableObjectBase, PrimitiveBase> > {
protected:
	std::string data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit String(std::string data);

	explicit String(const char* data);

	virtual PrimitiveBase* clone() const;

	virtual PrimitiveBase* plunder() &&;

	/**
	 * @return name of the symbol
	 */
	const std::string& getData() const;

	/**
	 * @return name of the symbol
	 */
	std::string& getData();

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const String& other) const;

	virtual void operator>>(std::ostream&) const;

	virtual explicit operator std::string () const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static String parse(std::deque<sax::Token>::iterator& input);
	static std::string parseRaw(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;
	static void compose(std::deque<sax::Token>& out, std::string primitive);

	String next() const;
	virtual void inc();
};

} /* namespace primitive */

namespace std {

template<>
struct compare<primitive::String> {
	int operator()(const primitive::String& first, const primitive::String& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

namespace alib{

template<>
struct xmlApi<std::string> {
	static std::string parse(std::deque<sax::Token>::iterator& input);
	static bool first(const std::deque<sax::Token>::const_iterator& input);
	static void compose(std::deque<sax::Token>& output, std::string data);
};

} /* namespace alib */

#endif /* PRIMITIVE_STRING_H_ */

