/*
 * Unsigned.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Unsigned.h"
#include "../sax/FromXMLParserHelper.h"
#include "Primitive.h"
#include "../object/Object.h"

namespace primitive {

Unsigned::Unsigned(unsigned data) : data(data) {

}

PrimitiveBase* Unsigned::clone() const {
	return new Unsigned(*this);
}

PrimitiveBase* Unsigned::plunder() && {
	return new Unsigned(std::move(*this));
}

unsigned Unsigned::getData() const {
	return data;
}

void Unsigned::setData(unsigned data) {
	this->data = data;
}

int Unsigned::compare(const Unsigned& other) const {
	return data - other.data;
}

void Unsigned::operator>>(std::ostream& out) const {
	out << "(Unsigned " << data << ")";
}

Unsigned::operator std::string() const {
	return std::to_string(data);
}

const std::string Unsigned::XML_TAG_NAME = "Unsigned";

Unsigned Unsigned::parse(std::deque<sax::Token>::iterator& input) {
	return Unsigned(parseRaw(input));
}

unsigned Unsigned::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Unsigned::XML_TAG_NAME);
	unsigned data = std::stoi(sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Unsigned::XML_TAG_NAME);
	return data;
}

void Unsigned::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Unsigned::compose(std::deque<sax::Token>& out, unsigned primitive) {
	out.emplace_back(Unsigned::XML_TAG_NAME, sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(std::itos(primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Unsigned::XML_TAG_NAME, sax::Token::TokenType::END_ELEMENT);
}

Unsigned Unsigned::next() const {
	return Unsigned(data + 1);
}

void Unsigned::inc() {
	data++;
}

} /* namespace primitive */

namespace alib {

unsigned xmlApi<unsigned>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Unsigned::parseRaw(input);
}

bool xmlApi<unsigned>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, primitive::Unsigned::XML_TAG_NAME);
}

void xmlApi<unsigned>::compose(std::deque<sax::Token>& output, unsigned data) {
	primitive::Unsigned::compose(output, data);
}

xmlApi<primitive::Primitive>::ParserRegister<primitive::Unsigned> unsignedParserRegister = xmlApi<primitive::Primitive>::ParserRegister<primitive::Unsigned>(primitive::Unsigned::XML_TAG_NAME, primitive::Unsigned::parse);
xmlApi<alib::Object>::ParserRegister<primitive::Unsigned> unsignedParserRegister2 = xmlApi<alib::Object>::ParserRegister<primitive::Unsigned>(primitive::Unsigned::XML_TAG_NAME, primitive::Unsigned::parse);

} /* namespace alib */

