/*
 * RegExpToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpToXMLComposer.h"

#include "../RegExp.h"
#include "../RegExpClasses.h"

#include "../unbounded/UnboundedRegExpElements.h"
#include "../formal/FormalRegExpElements.h"
#include "../../XmlApi.hpp"

namespace regexp {

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpAlternation& alternation) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("alternation", sax::Token::TokenType::START_ELEMENT);
	for (const auto& element : alternation.getElements()) {
		element->Accept(userData, *this);
	}
	out.emplace_back("alternation", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpConcatenation& concatenation) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("concatenation", sax::Token::TokenType::START_ELEMENT);
	for (auto element : concatenation.getElements()) {
		element->Accept(userData, *this);
	}
	out.emplace_back("concatenation", sax::Token::TokenType::END_ELEMENT);

}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpIteration& iteration) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("iteration", sax::Token::TokenType::START_ELEMENT);
	iteration.getElement().Accept(userData, *this);
	out.emplace_back("iteration", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpSymbol& symbol) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpEpsilon&) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("epsilon", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back("epsilon", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const UnboundedRegExpEmpty&) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("empty", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back("empty", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpAlternation& alternation) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("alternation", sax::Token::TokenType::START_ELEMENT);
	alternation.getLeftElement().Accept(userData, *this);
	alternation.getRightElement().Accept(userData, *this);
	out.emplace_back("alternation", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpConcatenation& concatenation) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("concatenation", sax::Token::TokenType::START_ELEMENT);
	concatenation.getLeftElement().Accept(userData, *this);
	concatenation.getRightElement().Accept(userData, *this);
	out.emplace_back("concatenation", sax::Token::TokenType::END_ELEMENT);

}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpIteration& iteration) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("iteration", sax::Token::TokenType::START_ELEMENT);
	iteration.getElement().Accept(userData, *this);
	out.emplace_back("iteration", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpSymbol& symbol) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	alib::xmlApi<alphabet::Symbol>::compose(out, symbol.getSymbol());
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpEpsilon&) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("epsilon", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back("epsilon", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::Visit(void* userData, const FormalRegExpEmpty&) const {
	std::deque<sax::Token> &out = *((std::deque<sax::Token>*) userData);

	out.emplace_back("empty", sax::Token::TokenType::START_ELEMENT);
	out.emplace_back("empty", sax::Token::TokenType::END_ELEMENT);
}

void RegExpToXMLComposer::composeAlphabet(std::deque<sax::Token>& out, const std::set<alphabet::Symbol>& alphabet) {
	out.emplace_back("alphabet", sax::Token::TokenType::START_ELEMENT);
	for (const auto& symbol : alphabet) {
		alib::xmlApi<alphabet::Symbol>::compose(out, symbol);
	}
	out.emplace_back("alphabet", sax::Token::TokenType::END_ELEMENT);
}

RegExpToXMLComposer RegExpToXMLComposer::REGEXP_TO_XML_COMPOSER;

} /* namespace regexp */
