/*
 * FormalRegExpEpsilon.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef FORMAL_REG_EXP_EPSILON_H_
#define FORMAL_REG_EXP_EPSILON_H_

#include "FormalRegExpElement.h"
#include <sstream>

namespace regexp {

/**
 * Represents epsilon in the regular expression.
 */
class FormalRegExpEpsilon: public std::acceptor<FormalRegExpEpsilon, FormalRegExpElement, FormalRegExpElement> {
protected:
	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* cloneAsUnbounded() const;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc FormalRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExpAlphabet * regexp );

	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:

	explicit FormalRegExpEpsilon();
	FormalRegExpEpsilon(const FormalRegExpEpsilon& other);
	FormalRegExpEpsilon(FormalRegExpEpsilon&& other) noexcept;
	FormalRegExpEpsilon& operator=(const FormalRegExpEpsilon& other);
	FormalRegExpEpsilon& operator=(FormalRegExpEpsilon&& other) noexcept;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement* clone() const;

	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement* plunder() &&;

	virtual int compare(const FormalRegExpElement& other) const {
		return FormalRegExpElement::compare(*this, other);
	}

	virtual int compare(const FormalRegExpEpsilon&) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	virtual explicit operator std::string() const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_EPSILON_H_ */
