/*
 * FormalRegExpIteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef FORMAL_REG_EXP_ITERATION_H_
#define FORMAL_REG_EXP_ITERATION_H_

#include "FormalRegExpElement.h"
#include "FormalRegExpEmpty.h"

namespace regexp {

namespace simplify {

class RegExpOptimize;

} /* namespace simplify */

/**
 * Represents iteration operator in the regular expression. Contains one FormalRegExpElement
 * as operand.
 */
class FormalRegExpIteration: public std::acceptor<FormalRegExpIteration, FormalRegExpElement, FormalRegExpElement> {
protected:
	FormalRegExpElement* element;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual UnboundedRegExpElement* cloneAsUnbounded() const;

	/**
	 * @copydoc FormalRegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc FormalRegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExpAlphabet * regexp );

	/**
	 * @copydoc FormalRegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit FormalRegExpIteration(FormalRegExpElement&&);
	explicit FormalRegExpIteration(const FormalRegExpElement&);

	FormalRegExpIteration(const FormalRegExpIteration& other);
	FormalRegExpIteration(FormalRegExpIteration&& other) noexcept;
	FormalRegExpIteration& operator =(const FormalRegExpIteration& other);
	FormalRegExpIteration& operator =(FormalRegExpIteration&& other) noexcept;
	virtual ~FormalRegExpIteration() noexcept;

	/**
	 * @copydoc FormalRegExpElement::clone() const
	 */
	virtual FormalRegExpElement* clone() const;

	/**
	 * @copydoc FormalRegExpElement::plunder() const
	 */
	virtual FormalRegExpElement* plunder() &&;


	/**
	 * @return element
	 */
	const FormalRegExpElement & getElement() const;

	/**
	 * @return element
	 */
	FormalRegExpElement & getElement();

	/**
	 * @param element to iterate
	 */
	void setElement(const FormalRegExpElement& element);

	void setElement(FormalRegExpElement&& element);

	virtual int compare(const FormalRegExpElement& other) const {
		return FormalRegExpElement::compare(*this, other);
	}

	virtual int compare(const FormalRegExpIteration&) const;

	/**
	 * @copydoc FormalRegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	friend class simplify::RegExpOptimize;

	virtual explicit operator std::string() const;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}
};

} /* namespace regexp */

#endif /* FORMAL_REG_EXP_ITERATION_H_ */
