/*
 * UnboundedRegExpEmpty.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "UnboundedRegExpEmpty.h"
#include "../formal/FormalRegExpEmpty.h"
#include <sstream>

namespace regexp {

UnboundedRegExpEmpty::UnboundedRegExpEmpty() {
	// so that default constructor is available
}

UnboundedRegExpEmpty::UnboundedRegExpEmpty(const UnboundedRegExpEmpty&) {
	// so that copy constructor is available
}

UnboundedRegExpEmpty::UnboundedRegExpEmpty(UnboundedRegExpEmpty&&) noexcept {
	this->attachRegExp(NULL);
}

UnboundedRegExpEmpty& UnboundedRegExpEmpty::operator =(const UnboundedRegExpEmpty&) {
	// this is actually different than default implementation
	return *this;
}

UnboundedRegExpEmpty& UnboundedRegExpEmpty::operator =(UnboundedRegExpEmpty&&) noexcept {
	// this is actually different than default implementation
	return *this;
}

UnboundedRegExpElement* UnboundedRegExpEmpty::clone() const {
	return new UnboundedRegExpEmpty(*this);
}

UnboundedRegExpElement* UnboundedRegExpEmpty::plunder() && {
	return new UnboundedRegExpEmpty(std::move(*this));
}

FormalRegExpElement* UnboundedRegExpEmpty::cloneAsFormal() const {
	return new FormalRegExpEmpty();
}

int UnboundedRegExpEmpty::compare(const UnboundedRegExpEmpty&) const {
	return 0;
}

void UnboundedRegExpEmpty::operator>>(std::ostream& out) const {
	out << "(UnboundedRegExpEmpty)";
}

bool UnboundedRegExpEmpty::testSymbol( const alphabet::Symbol & ) const {
	return false;
}

bool UnboundedRegExpEmpty::attachRegExp(const RegExpAlphabet * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	return true;
}

void UnboundedRegExpEmpty::computeMinimalAlphabet( std::set<alphabet::Symbol>&) const {

}

UnboundedRegExpEmpty::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */
