/*
 * UnboundedRegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "UnboundedRegExpSymbol.h"
#include "../formal/FormalRegExpSymbol.h"
#include <sstream>

namespace regexp {

UnboundedRegExpSymbol::UnboundedRegExpSymbol(int number) : symbol(alphabet::symbolFrom(number)) {

}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(char character) : symbol(alphabet::symbolFrom(character)) {

}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(const std::string& label) : symbol(alphabet::symbolFrom(label)) {

}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(const UnboundedRegExpSymbol& other) : symbol(other.symbol) {

}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(UnboundedRegExpSymbol&& other) noexcept : symbol(std::move(other.symbol)) {
	this->attachRegExp(NULL);
}

UnboundedRegExpSymbol& UnboundedRegExpSymbol::operator=(const UnboundedRegExpSymbol& other) {
	if (this == &other) {
		return *this;
	}

	*this = UnboundedRegExpSymbol(other);

	return *this;
}

UnboundedRegExpSymbol& UnboundedRegExpSymbol::operator=(UnboundedRegExpSymbol&& other) noexcept {
	std::swap(this->symbol, other.symbol);
	std::swap(this->parentRegExp, other.parentRegExp);

	this->attachRegExp(other.parentRegExp);

	return *this;
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(const alphabet::Symbol& symbol) :
		symbol(symbol) {
}

UnboundedRegExpSymbol::UnboundedRegExpSymbol(alphabet::Symbol&& symbol) :
		symbol(std::move(symbol)) {
}

UnboundedRegExpElement* UnboundedRegExpSymbol::clone() const {
	return new UnboundedRegExpSymbol(*this);
}

UnboundedRegExpElement* UnboundedRegExpSymbol::plunder() && {
	return new UnboundedRegExpSymbol(std::move(*this));
}

FormalRegExpElement* UnboundedRegExpSymbol::cloneAsFormal() const {
	return new FormalRegExpSymbol(this->symbol);
}

bool UnboundedRegExpSymbol::operator==(const alphabet::Symbol& other) const {
	return this->symbol == other;
}

bool operator==(const alphabet::Symbol& first, const UnboundedRegExpSymbol& second) {
	return first == second.symbol;
}

int UnboundedRegExpSymbol::compare(const UnboundedRegExpSymbol& other) const {
	return symbol.getData().compare(other.symbol.getData());
}

void UnboundedRegExpSymbol::operator>>(std::ostream& out) const {
	out << "(UnboundedRegExpSymbol " << symbol << ")";
}

bool UnboundedRegExpSymbol::testSymbol( const alphabet::Symbol & symbol ) const {
	return symbol == this->symbol;
}

bool UnboundedRegExpSymbol::attachRegExp(const RegExpAlphabet * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	if(regexp == NULL) return true;
	return this->parentRegExp->getAlphabet().find(this->symbol) != this->parentRegExp->getAlphabet().end();
}

void UnboundedRegExpSymbol::computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const {
	alphabet.insert(this->symbol);
}

const alphabet::Symbol& UnboundedRegExpSymbol::getSymbol() const {
	return this->symbol;
}

UnboundedRegExpSymbol::operator std::string() const {
	std::stringstream ss;
	ss << *this;
	return ss.str();
}

} /* namespace regexp */

