/*
 * Epsilon.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_H_
#define EPSILON_H_

#include "StringBase.h"
#include <iostream>
#include <set>
#include <vector>

#include "../alphabet/Symbol.h"
#include "common/StringAlphabet.h"

namespace string {

/**
 * Represents epsilon. Regular expression is stored
 * as a tree of EpsilonElement.
 */
class Epsilon : public std::acceptor<Epsilon, VisitableStringBase, std::acceptor<Epsilon, alib::VisitableObjectBase, StringBase> >, public StringAlphabet {
public:
	Epsilon();

	Epsilon(std::set<alphabet::Symbol> alphabet);

	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual const std::set<alphabet::Symbol>& getAlphabet() const {
		return StringAlphabet::getAlphabet();
	}

	virtual bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);

	/**
	 * @return List of symbols forming string (const version).
	 */
	std::vector<alphabet::Symbol> getContent() const;

	/**
	 * @return true
	 */
	bool isEmpty() const;

	virtual int compare(const ObjectBase& other) const {
		return ObjectBase::compare(*this, other);
	}

	virtual int compare(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual explicit operator std::string() const;

	static Epsilon EPSILON;

	virtual long long selfTypeId() const {
		return typeId(*this);
	}

	const static std::string XML_TAG_NAME;

	static Epsilon parse(std::deque<sax::Token>::iterator& input);

	void compose(std::deque<sax::Token>& out) const;
};

} /* namespace string */

namespace std {

template<>
struct compare<::string::Epsilon> {
	int operator()(const ::string::Epsilon& first, const ::string::Epsilon& second) const {
		return first.compare(second);
	}
};

} /* namespace std */

#endif /* EPSILON_H_ */

