/*
 * RankedAlphabet.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Stepan Plachy
 */

#include "RankedAlphabet.h"

#include <algorithm>

namespace tree {

const std::set<alphabet::RankedSymbol>& RankedAlphabet::getAlphabet() const {
	return alphabet;
}

bool RankedAlphabet::addSymbolToAlphabet(alphabet::RankedSymbol symbol) {
	return alphabet.insert(std::move(symbol)).second;
}

void RankedAlphabet::addSymbolsToAlphabet(const std::set<alphabet::RankedSymbol>& symbols) {
	for(const alphabet::RankedSymbol& addedSymbol : symbols) {
		addSymbolToAlphabet(addedSymbol);
	}
}

void RankedAlphabet::setAlphabet(std::set<alphabet::RankedSymbol> newSymbols) {
	std::set<alphabet::RankedSymbol> removed;
	std::set_difference(alphabet.begin(), alphabet.end(), newSymbols.begin(), newSymbols.end(), std::inserter(removed, removed.end()));

	for(const alphabet::RankedSymbol& removedSymbol : removed) {
		removeSymbolFromAlphabet(removedSymbol);
	}

	alphabet = std::move(newSymbols);
}

} /* namespace tree */

