/*
 * TreeFromXMLParser.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#include "../../sax/FromXMLParserHelper.h"
#include "TreeFromXMLParser.h"
#include "../../sax/ParserException.h"
#include "../../XmlApi.hpp"

namespace tree {

alphabet::Symbol TreeFromXMLParser::parseBar(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "bar");
	alphabet::Symbol bar(alib::xmlApi<alphabet::Symbol>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "bar");
	return bar;
}

alphabet::Symbol TreeFromXMLParser::parseVariablesBar(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "variablesBar");
	alphabet::Symbol variablesBar(alib::xmlApi<alphabet::Symbol>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "variablesBar");
	return variablesBar;
}

alphabet::Symbol TreeFromXMLParser::parseSubtreeWildcardSymbol(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "subtreeWildcard");
	alphabet::Symbol subtreeWildcard(alib::xmlApi<alphabet::Symbol>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "subtreeWildcard");
	return subtreeWildcard;
}

alphabet::RankedSymbol TreeFromXMLParser::parseSubtreeWildcardRankedSymbol(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "subtreeWildcard");
	alphabet::RankedSymbol subtreeWildcard(alib::xmlApi<alphabet::RankedSymbol>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "subtreeWildcard");
	return subtreeWildcard;
}

std::set<alphabet::RankedSymbol> TreeFromXMLParser::parseRankedAlphabet(std::deque<sax::Token>::iterator& input) {
	std::set<alphabet::RankedSymbol> rankedSymbols;
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "rankedAlphabet");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		rankedSymbols.insert(alib::xmlApi<alphabet::RankedSymbol>::parse(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "rankedAlphabet");
	return rankedSymbols;
}

std::set<alphabet::Symbol> TreeFromXMLParser::parseAlphabet(std::deque<sax::Token>::iterator& input) {
	std::set<alphabet::Symbol> symbols;
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "alphabet");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		symbols.insert(alib::xmlApi<alphabet::Symbol>::parse(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "alphabet");
	return symbols;
}

std::vector<alphabet::RankedSymbol> TreeFromXMLParser::parseContentData(std::deque<sax::Token>::iterator& input) {
	std::vector<alphabet::RankedSymbol> data;
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "content");
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		data.push_back(alib::xmlApi<alphabet::RankedSymbol>::parse(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "content");
	return data;
}

RankedNode * TreeFromXMLParser::parseRankedNode(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "rankedNode");
	alphabet::RankedSymbol symbol = alib::xmlApi<alphabet::RankedSymbol>::parse(input);
	int rank = symbol.getRank().getData();
	std::vector<RankedNode *> children;
	children.reserve(rank);
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		children.push_back(parseRankedNode(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "rankedNode");

	return new RankedNode(symbol, children);
}

UnrankedNode * TreeFromXMLParser::parseUnrankedNode(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, "unrankedNode");
	alphabet::Symbol symbol = alib::xmlApi<alphabet::Symbol>::parse(input);
	std::vector<UnrankedNode *> children;
	while (sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		children.push_back(parseUnrankedNode(input));
	}
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, "unrankedNode");

	return new UnrankedNode(symbol, children);
}

} /* namespace tree */

