#include <list>
#include "GrammarTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "grammar/Unrestricted/UnrestrictedGrammar.h"
#include "grammar/Unrestricted/ContextPreservingUnrestrictedGrammar.h"

#include "grammar/Regular/RightRG.h"
#include "grammar/Regular/RightLG.h"

#include "grammar/ContextFree/LG.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/CNF.h"
#include "grammar/ContextFree/GNF.h"

#include "grammar/ContextSensitive/CSG.h"
#include "grammar/ContextSensitive/NonContractingGrammar.h"

#include "factory/XmlDataFactory.hpp"

#include "alphabet/LabeledSymbol.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GrammarTest, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION( GrammarTest );

void GrammarTest::setUp() {
}

void GrammarTest::tearDown() {
}

void GrammarTest::testUnrestrictedParser() {
	grammar::UnrestrictedGrammar grammar(alphabet::symbolFrom(1));

	grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
	grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
	grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
	grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
	grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

	grammar.addRule({alphabet::symbolFrom(1)}, {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
	grammar.addRule({alphabet::symbolFrom(2)}, {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
	grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(2), alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
	grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
	grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(1)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)});
	grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(1)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
	grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {});

	CPPUNIT_ASSERT( grammar == grammar );
	{
		std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
		std::string tmp;
		sax::SaxComposeInterface::printMemory(tmp, tokens);

		std::deque<sax::Token> tokens2;
		sax::SaxParseInterface::parseMemory(tmp, tokens2);
		grammar::UnrestrictedGrammar grammar2 = alib::XmlDataFactory::fromTokens<grammar::UnrestrictedGrammar>(tokens2);

		CPPUNIT_ASSERT( grammar == grammar2 );
	}
	{
		grammar::ContextPreservingUnrestrictedGrammar grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)}, alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(2), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(2), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {});
		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::ContextPreservingUnrestrictedGrammar grammar2 = alib::XmlDataFactory::fromTokens<grammar::ContextPreservingUnrestrictedGrammar>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

void GrammarTest::testRegularParser() {
	{
		grammar::RightRG grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(alphabet::symbolFrom("a"), alphabet::symbolFrom(2)));
		grammar.addRule(alphabet::symbolFrom(2), std::make_pair(alphabet::symbolFrom("b"), alphabet::symbolFrom(3)));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::RightRG grammar2 = alib::XmlDataFactory::fromTokens<grammar::RightRG>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}

	{
		grammar::RightLG grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(std::vector<alphabet::Symbol> {alphabet::symbolFrom("a")}, alphabet::symbolFrom(2)));
		grammar.addRule(alphabet::symbolFrom(2), std::make_pair(std::vector<alphabet::Symbol> {alphabet::symbolFrom("b")}, alphabet::symbolFrom(3)));
		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(3)));
		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a")}, alphabet::symbolFrom(2)));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::cout << tmp << std::endl;

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::RightLG grammar2 = alib::XmlDataFactory::fromTokens<grammar::RightLG>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

void GrammarTest::testContextFreeParser() {
	{
		grammar::LG grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(alphabet::symbolFrom(1), std::make_tuple(std::vector<alphabet::Symbol> {alphabet::symbolFrom("a")}, alphabet::symbolFrom(2), std::vector<alphabet::Symbol>{}));
		grammar.addRule(alphabet::symbolFrom(2), std::make_tuple(std::vector<alphabet::Symbol> {alphabet::symbolFrom("b")}, alphabet::symbolFrom(3), std::vector<alphabet::Symbol>{}));
		grammar.addRule(alphabet::symbolFrom(1), std::make_tuple(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(3), std::vector<alphabet::Symbol>{}));
		grammar.addRule(alphabet::symbolFrom(1), std::make_tuple(std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a")}, alphabet::symbolFrom(2), std::vector<alphabet::Symbol>{}));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::LG grammar2 = alib::XmlDataFactory::fromTokens<grammar::LG>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::EpsilonFreeCFG grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));
		grammar.setGeneratesEpsilon(true);

		grammar.addRule(alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(alphabet::symbolFrom(2), std::vector<alphabet::Symbol> {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
		grammar.addRule(alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)});
		grammar.addRule(alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::EpsilonFreeCFG grammar2 = alib::XmlDataFactory::fromTokens<grammar::EpsilonFreeCFG>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::CFG grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(alphabet::symbolFrom(2), std::vector<alphabet::Symbol> {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
		grammar.addRule(alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)});
		grammar.addRule(alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(alphabet::symbolFrom(2), std::vector<alphabet::Symbol> {});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::CFG grammar2 = alib::XmlDataFactory::fromTokens<grammar::CFG>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::CNF grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));
		grammar.setGeneratesEpsilon(true);

		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(alphabet::symbolFrom(3), alphabet::symbolFrom(2)));
		grammar.addRule(alphabet::symbolFrom(2), std::make_pair(alphabet::symbolFrom(3), alphabet::symbolFrom(3)));
		grammar.addRule(alphabet::symbolFrom(3), alphabet::symbolFrom("a"));
		grammar.addRule(alphabet::symbolFrom(1), alphabet::symbolFrom("a"));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::CNF grammar2 = alib::XmlDataFactory::fromTokens<grammar::CNF>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::GNF grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));
		grammar.setGeneratesEpsilon(true);

		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(alphabet::symbolFrom("a"), std::vector<alphabet::Symbol> {alphabet::symbolFrom(2)}));
		grammar.addRule(alphabet::symbolFrom(2), std::make_pair(alphabet::symbolFrom("b"), std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)}));
		grammar.addRule(alphabet::symbolFrom(3), std::make_pair(alphabet::symbolFrom("a"), std::vector<alphabet::Symbol> {}));
		grammar.addRule(alphabet::symbolFrom(1), std::make_pair(alphabet::symbolFrom("a"), std::vector<alphabet::Symbol> {}));

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::GNF grammar2 = alib::XmlDataFactory::fromTokens<grammar::GNF>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

void GrammarTest::testContextSensitiveParser() {
	{
		grammar::CSG grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)}, alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(2), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)});
		grammar.addRule(std::vector<alphabet::Symbol> {}, alphabet::symbolFrom(1), std::vector<alphabet::Symbol> {}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::CSG grammar2 = alib::XmlDataFactory::fromTokens<grammar::CSG>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
	{
		grammar::NonContractingGrammar grammar(alphabet::symbolFrom(1));

		grammar.addNonterminalSymbol(alphabet::symbolFrom(1));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(2));
		grammar.addNonterminalSymbol(alphabet::symbolFrom(3));
		grammar.addTerminalSymbol(alphabet::symbolFrom("a"));
		grammar.addTerminalSymbol(alphabet::symbolFrom("b"));

		grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(2), alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});
		grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(2)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("b"), alphabet::symbolFrom(3)});
		grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(1)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom(3)});
		grammar.addRule(std::vector<alphabet::Symbol> {alphabet::symbolFrom(1)}, std::vector<alphabet::Symbol> {alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom("a"), alphabet::symbolFrom(2)});

		CPPUNIT_ASSERT( grammar == grammar );
		{
			std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(grammar);
			std::string tmp;
			sax::SaxComposeInterface::printMemory(tmp, tokens);

			std::deque<sax::Token> tokens2;
			sax::SaxParseInterface::parseMemory(tmp, tokens2);
			grammar::NonContractingGrammar grammar2 = alib::XmlDataFactory::fromTokens<grammar::NonContractingGrammar>(tokens2);

			CPPUNIT_ASSERT( grammar == grammar2 );
		}
	}
}

