/*
 * UsefullStates.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "UsefullStates.h"

#include <exception/AlibException.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <set>
#include <map>
#include <queue>

namespace automaton {

namespace properties {

namespace efficient {

std::set<automaton::State> UsefullStates::usefullStates(const Automaton& automaton) {
	std::set<automaton::State> out;
	automaton.getData().Accept((void*) &out, UsefullStates::USEFULL_STATES);
	return out;
}

template<class T>
std::set<automaton::State> UsefullStates::usefullStates( const T & fsm ) {
	std::map<automaton::State, std::set<automaton::State>> reversedTransitions;
	for(const auto& transition : fsm.getTransitions())
		for(const automaton::State& to : transition.second)
			reversedTransitions[to].insert(transition.first.first);

	std::deque<automaton::State> queue ( fsm.getFinalStates( ).begin(), fsm.getFinalStates().end() );
	std::set<automaton::State> visited = fsm.getFinalStates( );

	while( !queue.empty() ) {
		const std::set<automaton::State>& to = reversedTransitions[queue.front()];
		queue.pop_front();

		for(const automaton::State& process : to)
			if(visited.insert(process).second) {
				queue.push_back(std::move(const_cast<automaton::State&>(process)));
			}
	}

	return visited;
}

template std::set<automaton::State> UsefullStates::usefullStates( const automaton::EpsilonNFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::NFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::CompactNFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::ExtendedNFA & fsm );
template std::set<automaton::State> UsefullStates::usefullStates( const automaton::MultiInitialStateNFA & fsm );

template<>
std::set<automaton::State> UsefullStates::usefullStates( const automaton::DFA & fsm ) {
	std::map<automaton::State, std::set<automaton::State>> reversedTransitions;
	for(const auto& transition : fsm.getTransitions())
		reversedTransitions[transition.second].insert(transition.first.first);

	std::deque<automaton::State> queue ( fsm.getFinalStates( ).begin(), fsm.getFinalStates().end() );
	std::set<automaton::State> visited = fsm.getFinalStates( );

	while( !queue.empty() ) {
		const std::set<automaton::State>& to = reversedTransitions[queue.front()];
		queue.pop_front();

		for(const automaton::State& process : to)
			if(visited.insert(process).second) {
				queue.push_back(std::move(const_cast<automaton::State&>(process)));
			}
	}

	return visited;
}

void UsefullStates::Visit(void* data, const EpsilonNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const MultiInitialStateNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const NFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const DFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const ExtendedNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void* data, const CompactNFA& automaton) const {
	std::set<automaton::State> & out = *((std::set<automaton::State>*) data);
	out = std::move(this->usefullStates(automaton));
}

void UsefullStates::Visit(void*, const DFTA&) const {
	throw exception::AlibException("Unsupported automaton type DFTA");
}

void UsefullStates::Visit(void*, const NFTA&) const {
	throw exception::AlibException("Unsupported automaton type NFTA");
}

void UsefullStates::Visit(void*, const DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void UsefullStates::Visit(void*, const SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void UsefullStates::Visit(void*, const InputDrivenDPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void UsefullStates::Visit(void*, const InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void UsefullStates::Visit(void*, const VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void UsefullStates::Visit(void*, const VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void UsefullStates::Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void UsefullStates::Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void UsefullStates::Visit(void*, const NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void UsefullStates::Visit(void*, const SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void UsefullStates::Visit(void*, const OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const UsefullStates UsefullStates::USEFULL_STATES;

} /* namespace efficient */

} /* namespace properties */

} /* namespace automaton */

