/*
 * Trim.cpp
 *
 *  Created on: 23. 3. 2014
 *	  Author: Tomas Pecka
 */

#include "Trim.h"
#include "UselessStatesRemover.h"
#include "UnreachableStatesRemover.h"

#include <exception/AlibException.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <automaton/FSM/CompactNFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include "automaton/common/State.h"
#include "automaton/Automaton.h"

namespace automaton {

namespace simplify {

namespace efficient {

template<class T>
T Trim::trim( const T & fsm ) {
	return UnreachableStatesRemover::remove(UselessStatesRemover::remove(fsm));
}

template automaton::EpsilonNFA Trim::trim( const automaton::EpsilonNFA & fsm );
template automaton::NFA Trim::trim( const automaton::NFA & fsm );
template automaton::MultiInitialStateNFA Trim::trim( const automaton::MultiInitialStateNFA & fsm );
template automaton::CompactNFA Trim::trim( const automaton::CompactNFA & fsm );
template automaton::ExtendedNFA Trim::trim( const automaton::ExtendedNFA & fsm );
template automaton::DFA Trim::trim( const automaton::DFA & fsm );

automaton::Automaton Trim::trim(const automaton::Automaton& automaton) {
	automaton::Automaton* out = NULL;
	automaton.getData().Accept((void*) &out, Trim::TRIM);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

void Trim::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->trim(automaton));
}

void Trim::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->trim(automaton));
}

void Trim::Visit(void* data, const automaton::NFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->trim(automaton));
}

void Trim::Visit(void* data, const automaton::DFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->trim(automaton));
}

void Trim::Visit(void* data, const automaton::ExtendedNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->trim(automaton));
}

void Trim::Visit(void* data, const automaton::CompactNFA& automaton) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->trim(automaton));
}

void Trim::Visit(void*, const DFTA&) const {
	throw exception::AlibException("Unsupported automaton type DFTA");
}

void Trim::Visit(void*, const NFTA&) const {
	throw exception::AlibException("Unsupported automaton type NFTA");
}

void Trim::Visit(void*, const automaton::DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void Trim::Visit(void*, const automaton::SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void Trim::Visit(void*, const automaton::InputDrivenDPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void Trim::Visit(void*, const automaton::InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void Trim::Visit(void*, const automaton::VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void Trim::Visit(void*, const automaton::VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void Trim::Visit(void*, const automaton::RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void Trim::Visit(void*, const automaton::RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void Trim::Visit(void*, const automaton::NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void Trim::Visit(void*, const automaton::SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void Trim::Visit(void*, const automaton::OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const Trim Trim::TRIM;

} /* namespace efficient */

} /* namespace simplify */

} /* namespace automaton */

