/*
 * TreeToRawComposer.cpp
 *
 *  Created on: Nov 16, 2014
 *      Author: Jan Travnicek
 */

#include "TreeToRawComposer.h"
#include "tree/TreeException.h"
#include "tree/TreeClasses.h"

namespace tree {

void TreeToRawComposer::compose(std::deque<sax::Token>& out, const Tree& tree) {
	getInstance().dispatch(out, tree.getData());
}

void TreeToRawComposer::compose(std::deque<sax::Token>& out, const RankedTree& tree) {
	composeNode(out, tree.getRoot());
}

TreeToRawComposer::RegistratorWrapper<void, RankedTree> TreeToRawComposerRankedTree = TreeToRawComposer::RegistratorWrapper<void, RankedTree>(TreeToRawComposer::getInstance(), TreeToRawComposer::compose);

void TreeToRawComposer::compose(std::deque<sax::Token>& out, const PrefixRankedTree& tree) {
	unsigned i = 0;
	composePrefixRankedNotation(out, i, tree.getContent());
}

TreeToRawComposer::RegistratorWrapper<void, PrefixRankedTree> TreeToRawComposerPrefixRankedTree = TreeToRawComposer::RegistratorWrapper<void, PrefixRankedTree>(TreeToRawComposer::getInstance(), TreeToRawComposer::compose);

void TreeToRawComposer::compose(std::deque<sax::Token>& out, const PrefixRankedBarTree& tree) {
	unsigned i = 0;
	composePrefixRankedBarNotation(out, i, tree.getContent());
}

TreeToRawComposer::RegistratorWrapper<void, PrefixRankedBarTree> TreeToRawComposerPrefixRankedBarTree = TreeToRawComposer::RegistratorWrapper<void, PrefixRankedBarTree>(TreeToRawComposer::getInstance(), TreeToRawComposer::compose);

void TreeToRawComposer::compose(std::deque<sax::Token>& out, const UnrankedTree& tree) {
	composeNode(out, tree.getRoot());
}

TreeToRawComposer::RegistratorWrapper<void, UnrankedTree> TreeToRawComposerUnrankedTree = TreeToRawComposer::RegistratorWrapper<void, UnrankedTree>(TreeToRawComposer::getInstance(), TreeToRawComposer::compose);

void TreeToRawComposer::composeNode(std::deque<sax::Token>& out, const RankedNode& node) {
	if(node.getChildren().size() == 0) {
		out.emplace_back(sax::Token((std::string) node.getSymbol(), sax::Token::TokenType::CHARACTER));
	} else {
		out.emplace_back(sax::Token((std::string) node.getSymbol(), sax::Token::TokenType::START_ELEMENT));
		for(const auto& child : node.getChildren()) {
			composeNode(out, *child);
		}

		out.emplace_back(sax::Token((std::string) node.getSymbol(), sax::Token::TokenType::END_ELEMENT));
	}
}

void TreeToRawComposer::composePrefixRankedNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<alphabet::RankedSymbol>& notation) {
	unsigned size = notation[index].getRank().getData();
	unsigned root = index;
	if(size == 0) {
		out.emplace_back(sax::Token((std::string) notation[root], sax::Token::TokenType::CHARACTER));
		index += 1;
	} else {
		out.emplace_back(sax::Token((std::string) notation[root], sax::Token::TokenType::START_ELEMENT));

		index += 1;
		for(unsigned i = 0; i < size; ++i) {
			composePrefixRankedNotation(out, index, notation);
		}

		out.emplace_back(sax::Token((std::string) notation[root], sax::Token::TokenType::END_ELEMENT));
	}
}

void TreeToRawComposer::composePrefixRankedBarNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<alphabet::RankedSymbol>& notation) {
	unsigned size = notation[index].getRank().getData();
	unsigned root = index;
	if(size == 0) {
		out.emplace_back(sax::Token((std::string) notation[root], sax::Token::TokenType::CHARACTER));
		index += 2;
	} else {
		out.emplace_back(sax::Token((std::string) notation[root], sax::Token::TokenType::START_ELEMENT));

		index += 1;
		for(unsigned i = 0; i < size; i++) {
			composePrefixRankedBarNotation(out, index, notation);
		}
		index += 1;

		out.emplace_back(sax::Token((std::string) notation[root], sax::Token::TokenType::END_ELEMENT));
	}
}

void TreeToRawComposer::composeNode(std::deque<sax::Token>& out, const UnrankedNode& node) {
	if(node.getChildren().size() == 0) {

	} else {
		out.emplace_back(sax::Token((std::string) node.getSymbol(), sax::Token::TokenType::START_ELEMENT));
		for(const auto& child : node.getChildren()) {
			composeNode(out, *child);
		}

		out.emplace_back(sax::Token((std::string) node.getSymbol(), sax::Token::TokenType::END_ELEMENT));
	}
}

} /* namespace automaton */

