/*
 * TreeToRawComposer.h
 *
 *  Created on: Nov 16, 2014
 *      Author: Jan Travnicek
 */

#ifndef TREE_TO_RAW_COMPOSER_H_
#define TREE_TO_RAW_COMPOSER_H_

#include <deque>
#include <common/multipleDispatch.hpp>
#include "tree/Tree.h"
#include "tree/TreeFeatures.h"
#include "alphabet/SymbolFeatures.h"
#include "sax/Token.h"
#include <vector>

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToRawComposer : public std::SingleDispatchFirstStaticParam<void, std::deque<sax::Token>, TreeBase> {
public:
	static void compose(std::deque<sax::Token>& out, const Tree& tree);

	static void compose(std::deque<sax::Token>& out, const RankedTree& tree);
	static void compose(std::deque<sax::Token>& out, const PrefixRankedTree& tree);
	static void compose(std::deque<sax::Token>& out, const PrefixRankedBarTree& tree);
	static void compose(std::deque<sax::Token>& out, const UnrankedTree& tree);

private:
	static void composeNode(std::deque<sax::Token>& out, const RankedNode& node);
	static void composePrefixRankedNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<alphabet::RankedSymbol>& notation);
	static void composePrefixRankedBarNotation(std::deque<sax::Token>& out, unsigned& index, const std::vector<alphabet::RankedSymbol>& notation);
	static void composeNode(std::deque<sax::Token>& out, const UnrankedNode& node);

public:
	static TreeToRawComposer& getInstance() {
		static TreeToRawComposer res;
		return res;
	}
};

} /* namespace tree */

#endif /* TREE_TO_RAW_COMPOSER_H_ */

