/*
 * chrono.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#include "../chrono"

namespace std {

namespace chrono {

measurements measurements::INSTANCE;

measurements::Format measurements::Format::LIST = { 0 };
measurements::Format measurements::Format::TREE = { 1 };

measurements::Filter measurements::Filter::ALL = { true, ~0u };
measurements::Filter measurements::Filter::NONE = { false, ~0u };
measurements::Filter measurements::Filter::OVERALL = { true, measurements::Type::OVERALL };
measurements::Filter measurements::Filter::NO_OVERALL = { false, measurements::Type::OVERALL };
measurements::Filter measurements::Filter::INIT = { true, measurements::Type::INIT };
measurements::Filter measurements::Filter::NO_INIT = { false, measurements::Type::INIT };
measurements::Filter measurements::Filter::FINALIZE = { true, measurements::Type::FINALIZE };
measurements::Filter measurements::Filter::NO_FINALIZE = { false, measurements::Type::FINALIZE };
measurements::Filter measurements::Filter::MAIN = { true, measurements::Type::MAIN };
measurements::Filter measurements::Filter::NO_MAIN = { false, measurements::Type::MAIN };
measurements::Filter measurements::Filter::AUXILARY = { true, measurements::Type::AUXILARY };
measurements::Filter measurements::Filter::NO_AUXILARY = { false, measurements::Type::AUXILARY };

const int measurements::measurements_format_index = std::ios::xalloc();
const int measurements::measurements_filter_index = std::ios::xalloc();

void printAsList(std::ostream& out, const measurements::Results& data, unsigned index) {
	const unsigned filter = out.iword(measurements::measurements_filter_index);

	if(data.results.size() - 1 != index) out << ", ";
	if((std::get<1>(data[index]) & filter) == 0) out << data[index];
	for(unsigned idx : std::get<4>(data[index])) {
		printAsList(out, data, idx);
	}
}

void subtreeSizes(std::ostream& out, const measurements::Results& data, unsigned index, std::map<unsigned, unsigned>& res) {
	res[index] = 0;

	const unsigned filter = out.iword(measurements::measurements_filter_index);
	if((std::get<1>(data[index]) & filter) == 0) res[index] += 1;

	for(unsigned idx : std::get<4>(data[index])) {
		subtreeSizes(out, data, idx, res);
		res[index] += res[idx];
	}
}

void printAsTree(std::ostream& out, std::string& prefix, map<unsigned, unsigned>& indexToSize, const measurements::Results& data, unsigned index, bool last) {
	const unsigned filter = out.iword(measurements::measurements_filter_index);
	if((std::get<1>(data[index]) & filter) == 0) {
		if(last) prefix[prefix.size() - 2] = '\\';
		if(indexToSize[index] != 1) prefix += "+";

		out << prefix << data[index] << std::endl;

		if(indexToSize[index] == 1) prefix += "-";
		prefix += "-";
		if(last) prefix[prefix.size() - 4] = ' ';
		prefix[prefix.size() - 3] = ' ';
		prefix[prefix.size() - 2] = '|';

		unsigned size = std::get<4>(data[index]).size();
		for(unsigned idx : std::get<4>(data[index])) {
			printAsTree(out, prefix, indexToSize, data, idx, size == 1);
			size--;
		}
		prefix[prefix.size() - 3] = '-';

		prefix.pop_back();
		prefix.pop_back();
	} else {
		unsigned size = std::get<4>(data[index]).size();
		for(unsigned idx : std::get<4>(data[index])) {
			printAsTree(out, prefix, indexToSize, data, idx, size == 1 && last);
			size --;
		}
	}
}

std::ostream& operator<<(std::ostream& out, const measurements::Results& data) {
	const unsigned format = out.iword(measurements::measurements_format_index);

	unsigned index = data.results.size() - 1;
	if(format == measurements::Format::LIST.type) {
		printAsList(out, data, index);
	} else {
		std::string prefix = "|-";
		std::map<unsigned, unsigned> indexToSize;
		subtreeSizes(out, data, index, indexToSize);

		printAsTree(out, prefix, indexToSize, data, index, true);
	}

//	out << format << " " << filter << " " << data.results;
	return out;
}

std::ostream& operator<<(std::ostream& out, microseconds time) {
	out << time.count() << "ms";
	return out;
}

std::ostream& operator<<(std::ostream& out, measurements::Type type) {
	switch(type) {
	case measurements::Type::INIT:
		out << "INIT";
		break;
	case measurements::Type::FINALIZE:
		out << "FINALIZE";
		break;
	case measurements::Type::MAIN:
		out << "MAIN";
		break;
	case measurements::Type::AUXILARY:
		out << "AUXILARY";
		break;
	case measurements::Type::OVERALL:
		out << "OVERALL";
		break;
	}
	return out;
}

} /* namespace chrono */

} /* namespace std */

