/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef GRAPH_FROM_STRING_PARSER_H_
#define GRAPH_FROM_STRING_PARSER_H_

#include "graph/GraphFeatures.h"
#include "GraphFromStringLexer.h"

#include "graph/GraphRepresentation.h"
#include "graph/common/Node.h"

namespace alib {

template<typename T>
struct stringApi;

} // namespace alib

namespace graph {

class GraphFromStringParser
{
public:
	GraphFromStringParser() {}

private:
	Graph parseGraph(std::istream &input) const;
	DirectedGraph parseDirectedGraph(std::istream &input) const;
	UndirectedGraph parseUndirectedGraph(std::istream &input) const;

	REPRESENTATION parseRepresentation(std::istream &input) const;
	Node parseNode(std::istream &input) const;
	DirectedEdge parseDirectedEdge(std::istream &input) const;
	UndirectedEdge parseUndirectedEdge(std::istream &input) const;

	template<typename T>
	void parseNodes(std::istream &input, T &graph) const;
	void parseDirectedEdges(std::istream &input, DirectedGraph &graph) const;
	void parseUndirectedEdges(std::istream &input, UndirectedGraph &graph) const;
	void parseDelimiter(std::istream &input) const;
	int parseValue(std::istream &input) const;

	template<typename T>
	void parseNodeValues(std::istream &input, T &graph) const;
	void parseDirectedEdgeValues(std::istream &input, DirectedGraph &graph) const;
	void parseUndirectedEdgeValues(std::istream &input, UndirectedGraph &graph) const;

	GraphFromStringLexer lexer;

	template<typename T> friend struct alib::stringApi;
};

} // namespace graph

#endif // GRAPH_FROM_STRING_PARSER_H_
