/*
 * GraphFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: David Rosca
 */

#ifndef GRAPH_TO_STRING_COMPOSER_H_
#define GRAPH_TO_STRING_COMPOSER_H_

#include <set>
#include <common/multipleDispatch.hpp>
#include "graph/Graph.h"
#include "graph/GraphFeatures.h"
#include "graph/GraphRepresentation.h"
#include "graph/common/Node.h"
#include "graph/directed/DirectedEdge.h"
#include "graph/undirected/UndirectedEdge.h"

namespace graph {

class GraphToStringComposer : public std::SingleDispatchFirstStaticParam<void, std::ostream, GraphBase> {
private:
	static void composeNode(std::ostream& out, const Node &node);
	static void composeEdge(std::ostream& out, const DirectedEdge &edge);
	static void composeEdge(std::ostream& out, const UndirectedEdge &edge);

	static void composeRepresentation(std::ostream &out, REPRESENTATION representation);
	static void composeNodes(std::ostream &out, const std::set<Node> &nodes);
	static void composeDirectedEdges(std::ostream &out, const std::set<DirectedEdge> &edges);
	static void composeUndirectedEdges(std::ostream &out, const std::set<UndirectedEdge> &edges);

	template<typename T>
	static void composeNodeValues(std::ostream &out, const T &graph);
	template<typename T>
	static void composeEdgeValues(std::ostream &out, const T &graph);

public:
	static void compose(std::ostream& out, const Graph& graph);

	static void compose(std::ostream& out, const DirectedGraph& graph);
	static void compose(std::ostream& out, const UndirectedGraph& graph);

	static GraphToStringComposer& getInstance() {
		static GraphToStringComposer res;
		return res;
	}
};

} // namespace graph

#endif // GRAPH_TO_STRING_COMPOSER_H_
