/*
 * StringToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringToStringComposer.h"
#include "string/CyclicString.h"
#include "string/LinearString.h"

#include "../StringApi.hpp"

namespace string {

void StringToStringComposer::compose(std::ostream& out, const CyclicString& string) {
	out << "<";
	for(const auto& symbol : string.getContent()) {
		alib::stringApi<alphabet::Symbol>::compose(out, symbol);
	}
	out << ">";
}

StringToStringComposer::RegistratorWrapper<void, CyclicString> StringToStringComposerCyclicString = StringToStringComposer::RegistratorWrapper<void, CyclicString>(StringToStringComposer::getInstance(), StringToStringComposer::compose);

void StringToStringComposer::compose(std::ostream& out, const LinearString& string) {
	out << "\"";
	for(const auto& symbol : string.getContent()) {
		alib::stringApi<alphabet::Symbol>::compose(out, symbol);
	}
	out << "\"";
}

StringToStringComposer::RegistratorWrapper<void, LinearString> StringToStringComposerLinearString = StringToStringComposer::RegistratorWrapper<void, LinearString>(StringToStringComposer::getInstance(), StringToStringComposer::compose);

void StringToStringComposer::compose(std::ostream& out, const Epsilon&) {
	out << "#E";
}

StringToStringComposer::RegistratorWrapper<void, Epsilon> StringToStringComposerEpsilon = StringToStringComposer::RegistratorWrapper<void, Epsilon>(StringToStringComposer::getInstance(), StringToStringComposer::compose);

void StringToStringComposer::compose(std::ostream& out, const String& string) {
	getInstance().dispatch(out, string.getData());
}

} /* namespace string */
