/*
 * arename.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>

#include "exception/AlibException.h"
#include "factory/XmlDataFactory.hpp"
#include "automaton/simplify/Rename.h"

int main(int argc, char** argv) {

	try {
		TCLAP::CmdLine cmd("Automaton rename binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to rename",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		std::chrono::measurements::start("Overal", std::chrono::measurements::Type::OVERALL);
		std::chrono::measurements::start("Input read", std::chrono::measurements::Type::AUXILARY);

		std::deque<sax::Token> tokens;
		if(input.isSet()) {
			if(input.getValue() == "-") {
				sax::SaxParseInterface::parseStdin(tokens);
			} else {
				sax::SaxParseInterface::parseFile(input.getValue(), tokens);
			}
		} else {
			sax::SaxParseInterface::parseStdin(tokens);
		}

		automaton::Automaton automaton = alib::XmlDataFactory::fromTokens<automaton::Automaton>(tokens);

		std::chrono::measurements::end();
		std::chrono::measurements::start("Algorithm", std::chrono::measurements::Type::MAIN);

		automaton::Automaton res = automaton::simplify::Rename::rename(automaton);

		std::chrono::measurements::end();
		std::chrono::measurements::start("Output write", std::chrono::measurements::Type::AUXILARY);

		alib::XmlDataFactory::toStdout(res);

		std::chrono::measurements::end();
		std::chrono::measurements::end();

		if(measure.getValue()) std::clog << std::chrono::measurements::results() << std::endl;

		return 0;
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
