/*
 * AutomataStat.cpp
 *
 *  Created on: 20. 9. 2014
 *	  Author: Jan Travnicek
 */

#include "AutomataStat.h"

#include <exception/AlibException.h>

#include <automaton/FSM/NFA.h>
#include <automaton/FSM/DFA.h>

#include <factory/XmlDataFactory.hpp>
#include <container/ObjectsSet.h>
#include <container/ObjectsMap.h>
#include <iostream>

void AutomataStat::stat(const automaton::Automaton& automaton, const AutomataSettings& settings) {
	automaton.getData().Accept((void*) &settings, AutomataStat::AUTOMATA_STAT);
}

void AutomataStat::stat(const automaton::NFA& automaton, const AutomataSettings& settings) {
	switch(settings.states) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.finalStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getFinalStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getFinalStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.initialStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << 1 << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << 1 << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.inputAlphabet) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.transitions) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getTransitions().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getTransitions().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::NOOP:
		break;
	}
}

void AutomataStat::stat(const automaton::DFA& automaton, const AutomataSettings& settings) {
	switch(settings.states) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.finalStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getFinalStates().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getFinalStates().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getFinalStates());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.initialStates) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << 1 << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << 1 << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInitialState());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.inputAlphabet) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getInputAlphabet().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getInputAlphabet());
		break;
	case PrintingOptions::NOOP:
		break;
	}

	switch(settings.transitions) {
	case PrintingOptions::PRINT:
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::QUANTITY:
		std::cout << automaton.getTransitions().size() << std::endl;
		break;
	case PrintingOptions::BOTH:
		std::cout << automaton.getTransitions().size() << std::endl;
		alib::XmlDataFactory::toStdout(automaton.getTransitions());
		break;
	case PrintingOptions::NOOP:
		break;
	}
}

void AutomataStat::Visit(void*, const automaton::EpsilonNFA&) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void AutomataStat::Visit(void*, const automaton::MultiInitialStateNFA&) const {
	throw exception::AlibException("Unsupported automaton type EpsilonNFA");
}

void AutomataStat::Visit(void* data, const automaton::NFA& automaton) const {
	const AutomataSettings& in = *((const AutomataSettings*) data);
	this->stat(automaton, in);
}

void AutomataStat::Visit(void* data, const automaton::DFA& automaton) const {
	const AutomataSettings& in = *((const AutomataSettings*) data);
	this->stat(automaton, in);
}

void AutomataStat::Visit(void*, const automaton::ExtendedNFA&) const {
	throw exception::AlibException("Unsupported automaton type ExtendedNFA");
}

void AutomataStat::Visit(void*, const automaton::CompactNFA&) const {
	throw exception::AlibException("Unsupported automaton type CompactNFA");
}

void AutomataStat::Visit(void*, const automaton::DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDAAutomaton");
}

void AutomataStat::Visit(void*, const automaton::SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void AutomataStat::Visit(void*, const automaton::InputDrivenDPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenDPDA");
}

void AutomataStat::Visit(void*, const automaton::InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void AutomataStat::Visit(void*, const automaton::VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void AutomataStat::Visit(void*, const automaton::VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void AutomataStat::Visit(void*, const automaton::RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void AutomataStat::Visit(void*, const automaton::RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void AutomataStat::Visit(void*, const automaton::NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void AutomataStat::Visit(void*, const automaton::SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void AutomataStat::Visit(void*, const automaton::OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

void AutomataStat::Visit(void*, const automaton::DFTA&) const {
	throw exception::AlibException("Unsupported automaton type DFTA");
}

void AutomataStat::Visit(void*, const automaton::NFTA&) const {
	throw exception::AlibException("Unsupported automaton type NFTA");
}

const AutomataStat AutomataStat::AUTOMATA_STAT;

