#!/usr/bin/env bash

# $1 test dir suffix (debug / release)

# SETTINGS
TESTCASE_TIMEOUT=10
LOGFILE="log_tests.txt"

EXECUTABLES="aepsilon2 atrim2 adeterminize2 aminimize2 anormalize2 acompare2"
TESTS_DIR="`pwd`/examples2/automaton"

RES_GOOD=0
RES_FAIL=0
RES_TIME=0
RES_SEGV=0
RES_UNKN=0


# ----------------------------

for FILE in $EXECUTABLES; do
	if [ ! -f bin-$1/$FILE ]; then
		echo "Executable $FILE is required for testing. Make sure it is in bin-$1 folder."
		exit 1
	fi
done

cd bin-$1/
rm -f $LOGFILE

# ----------------------------

# $1 = conv command
# $2 = return code
# $3 = input automaton
# $4 = output of conversion
function log {
	echo "----------------------------------------------------------" >> $LOGFILE
	echo "conv: " $1 >> $LOGFILE
	echo "ret: " $2 >> $LOGFILE
	echo "input automaton:" >> $LOGFILE
	cat  "$3" >> $LOGFILE
	echo "command out:" >> $LOGFILE
	echo "$4" >> $LOGFILE
}

# $1 = what is beeing done (name)
# $2 = command for conversion. Output of such command must be (eps-)NFA !!
# $3 = automaton
# $4 = automaton to compare with
function runTest2 {
	OUT=`timeout $TESTCASE_TIMEOUT bash -c "./acompare2 <(cat $3 | $2 ) $4"`
	RET=$?

	if [ $RET == 0 ]; then # ok
		return 0
	fi

	log "$1" $RET "$3" "$OUT"

	if [ $RET == 124 ]; then # timeout
		return 2
	elif [ $RET -ge 124 ]; then #segv
		return 3
	else
		return 1
	fi
}

function registerResult {
	case $1 in
		0)
			echo -n "."
			((RES_GOOD++))
			;;
		1)
			echo -n "x"
			((RES_FAIL++))
			;;
		2)
			echo -n "T"
			((RES_TIME++))
			;;
		3)
			echo -n "E"
			((RES_SEGV++))
			;;
		*)
			echo -n "?"
			((RES_UNKN++))
			;;
	esac
}

function clearResults {
	RES_GOOD=0
	RES_FAIL=0
	RES_TIME=0
	RES_SEGV=0
	RES_UNKN=0
}

function outputResults {
	# summary
	echo -ne "\n\t"
	echo "RES: GOOD:" $RES_GOOD ", FAIL:" $RES_FAIL ", TIME:" $RES_TIME ", SEGV:" $RES_SEGV, "UNKN:" $RES_UNKN
	echo ""
}

# $1 - aconversions2 sequence
function runTest {
	echo $1 \($2\)
	echo -ne "\t"

	clearResults

	# predefined tests first
	for FILE in `ls $TESTS_DIR/$1*.xml | grep -v DET`; do
		DET_FILE=${FILE%.xml}.DET.xml
		if [ -f ${DET_FILE} ]; then
			runTest2 $1 "$2" "$FILE" "$DET_FILE"
			registerResult $?
		fi
	done

	outputResults
}

runTest "NFSM" "./adeterminize2 | ./atrim2"
runTest "ENFSM" "./aepsilon2 | ./adeterminize2"
runTest "NIDPDA" "./adeterminize2"
runTest "NPDA" "./adeterminize2 | ./anormalize2 --labels automaton"
runTest "NVPA" "./adeterminize2"
runTest "RHDPDA" "./adeterminize2"
runTest "NFTA" "./adeterminize2"
